
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.comm;

/**
 * Thrown when a bus or hardware error occurs with the CAN controller.
 */
public class CanBusException
   extends java.lang.Exception
{
   public static final int NONE              = 0;
   public static final int BIT_STUFF         = 1;
   public static final int FORMAT            = 2;
   public static final int TRANSMIT_NO_ACK   = 3;
   public static final int BIT_ONE           = 4;
   public static final int BIT_ZERO          = 5;
   public static final int CRC               = 6;
   public static final int COUNT_EXCEEDED    = 7;
   public static final int PORT_DISABLED     = 8;
   public static final int PORT_NOTOPENED    = 9;
   public static final int OPEN_ALREADYOPEN  = 10;
   public static final int CLOSE_NOTOWNER    = 11;
   public static final int ALLOCATION_ERROR  = 12;

   private int reason = NONE;

   /**
    * Constructs a <code>CanBusException</code> with no detail
    * message.
    */
   public CanBusException ()
   {
      super();
   }

  /**
    * Constructs a <code>CanBusException</code> with the specified
    * detail message and reason code.
    *
    * @param   s   the detail message
    * @param   reason  reason code for exception
    */
   public CanBusException (String s,int reason)
   {
      super(s);
      this.reason = reason;
   }

  /**
    * Returns an <code>int</code> representing the CAN Bus error that occurred.
    *
    * @return reason code for the exception
    */
   public int getReason ()
   {
      return reason;
   }
}
