
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.comm;



import com.dalsemi.system.*;


/**
 * <p>This class sends data to the debug port. This port is shared with the internal
 * 1-Wire port.  The methods in this class only call methods in the 
 * <code>com.dalsemi.system.Debug</code> class, but this class allows the 
 * debug port to be accessed as an <code>OutputStream</code>.  Special hardware
 * is required to access the debug port.  See <a href="http://www.ibutton.com/TINI/developers/socket72.pdf">
 * http://www.ibutton.com/TINI/developers/socket72.pdf</a> for details on 
 * board pinout for TINI and the components required to use
 * the debug port.</p>
 * 
 * <p>The debug port has 3 pins. Pin-1 is ground, 
 * pin-2 is for transmit data from the terminal (PC) and pin-3 is the 
 * data out from TINI. </p>
 * 
 * @see com.dalsemi.system.Debug
 */
public class DebugOutputStream
   extends java.io.OutputStream
{

   /**
    * Writes the specified byte to the debug port.  This method calls
    * the <code>com.dalsemi.system.Debug.debugDump(int)</code> method.
    *
    * @param b data to be written
    *
    * @throws IOException if an error occurs while writing to the debug port
    *
    * @see com.dalsemi.system.Debug#debugDump(int)
    */
   public void write (int b)
      throws java.io.IOException
   {
      Debug.debugDump(b);
   }

   /**
    * Writes the specified array to the debug port.  This method calls
    * the <code>com.dalsemi.system.Debug.debugDump(byte[],int)</code> method
    * if the argument <code>offset</code> is zero, but otherwise makes
    * <code>length</code> calls to the <code>write(int)</code> method.
    * For best performance, call with an <code>offset</code> of zero.
    * 
    * @param barr array containing data to be written
    * @param offset offset into array
    * @param length number of bytes to be written
    *
    * @throws IOException if an error occurs while writing to the debug port
    *
    * @see #write(int)
    * @see com.dalsemi.system.Debug#debugDump(byte[],int)
    */
   public void write (byte[] barr, int offset, int length)
      throws java.io.IOException
   {
      if (offset == 0)
         Debug.debugDump(barr, length);
      else
      {
         for (int i = 0; i < length; i++)
            write(barr [i + offset]);
      }
   }
}
