
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

/**
 * This class contains the interface to the native io methods.
 * CHANGES : 7/30/99 - CW - Made all blocking functions call through the ioctl interface
 */
package com.dalsemi.comm;



/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
class NativeComm
{

   // These values should match Clayton's IO manager code.

   /** Field STREAM_COMMAND           */
   public static final int STREAM_COMMAND = 0;

   /** Field STREAM_STDIN           */
   public static final int STREAM_STDIN = 1;

   /** Field STREAM_STDOUT           */
   public static final int STREAM_STDOUT = 2;

   /** Field STREAM_STDERR           */
   public static final int STREAM_STDERR = 3;

   /** Field STREAM_GENERIC           */
   public static final int  STREAM_GENERIC = 4;
   private static final int WRITE          = 0;
   private static final int READ           = 1;
   private static final int IOCTL          = 2;

   // These values should match Clayton's IO manager code.

   /** Field PORT_SERIAL0           */
   public static final int PORT_SERIAL0 = 0;

   /** Field PORT_SERIAL1           */
   public static final int PORT_SERIAL1 = 1;

   /** Field PORT_LCD           */
   public static final int PORT_LCD = 2;

   /** Field PORT_SERIAL2           */
   public static final int PORT_SERIAL2 = 3;

   /** Field PORT_SERIAL3           */
   public static final int PORT_SERIAL3 = 4;

   /**
    * Opens an IO port.  Calls the native code.
    *
    *
    * @param port - port number to open.
    * @param stream - stream type to open. (STDIN, STDOUT, STDERR)
    *
    * @return - handle to opened port.
    *
    * @see
    */
   private static native int io_openNative (int port, int stream);

   /**
    * Closes a previously opened port.
    *
    *
    * @param handle - the handle returned from a call to open.
    *
    * @return - success/failure
    *
    * @see
    */
   private static native int io_closeNative (int handle);

   private static native void setHandleNative (int handle);

   /**
    * Method setHandle
    *
    *
    * @param handle
    *
    */
   public static void setHandle (int handle)
   {
      setHandleNative(handle);
   }

   // private static native int io_writeNative(int handle,byte[] arr,int length);
   // private static native int io_readNative(int handle,byte[] arr,int length,int timeout);

   /**
    * Main method for calling all blocking IO functions.  Read/Write/IOCTL all block.
    *
    *
    * @param functionNumber - Number of the function to call.
    * @param handle - handle to an open port
    * @param arr - array to be passed to the driver
    * @param length - the length of the data in the array
    * @param timeout - the timeout value.
    *
    * @return
    *
    * @see
    */
   private static native int io_ioctlNative (int functionNumber, int handle,
                                             byte[] arr, int offset,
                                             int length, int timeout,
                                             boolean suspend);

   /**
    * Returns the number of available bytes that a driver has.
    *
    *
    * @param handle - handle to an open method.
    *
    * @return - number of bytes available in the driver.
    *
    * @see
    */
   private static native int io_availableNative (int handle);

   /**
    * opens a port for use.
    *
    *
    * @param port - The port type to open.
    * @param stream - The stream type to open.
    *
    * @return - handle to the opened port.
    *
    * @see
    */
   protected static int open (int port, int stream)
   {
      return (io_openNative(port, stream));
   }

   /**
    * Closes a previously opened port.
    *
    *
    * @param handle - handle of a previously opened port.
    *
    * @return - success/failure(0)
    *
    * @see
    */
   protected static int close (int handle)
   {
      return (io_closeNative(handle));
   }

   /**
    * Calls the drivers write method.
    *
    *
    * @param handle
    * @param arr
    *
    * @see
    */
   protected static void write (int handle, byte[] arr)
   {

      //com.dalsemi.system.Debug.debugDump(0x55);
      //com.dalsemi.system.Debug.debugDump(arr.length);
      if (arr.length != 0)
         io_ioctlNative(WRITE, handle, arr, 0, arr.length, 0x3FFFFFFF, true);

      //com.dalsemi.system.Debug.debugDump(0x56);
   }

   /**
    * Calls the drivers write method.
    *
    *
    * @param handle
    * @param arr
    * @param length
    *
    * @see
    */
   protected static void write (int handle, byte[] arr, int offset,
                                int length)
   {

      //com.dalsemi.system.Debug.debugDump(0x57);
      //com.dalsemi.system.Debug.debugDump(offset);
      if (length != 0)
         io_ioctlNative(WRITE, handle, arr, offset, length, 0x3FFFFFFF, true);

      //com.dalsemi.system.Debug.debugDump(0x58);
   }

   /**
    * Calls the drivers read method.
    *
    *
    * @param handle
    * @param arr
    * @param timeout
    *
    * @return
    *
    * @see
    */
   protected static int read (int handle, byte[] arr, int timeout,
                              boolean suspend)
   {   // com.dalsemi.jellybeans.Debug.debugDump(0x59);
      if (arr.length != 0)
         return (io_ioctlNative(READ, handle, arr, 0, arr.length, timeout,
                                !suspend));
      else
         return 0;
   }

   /**
    * Calls the drivers read method.
    *
    *
    * @param handle
    * @param arr
    * @param length
    * @param timeout
    * @param suspend - false - never times out.
    * @return
    *
    * @see
    */
   protected static int read (int handle, byte[] arr, int offset, int length,
                              int timeout, boolean suspend)
   {

      // com.dalsemi.jellybeans.Debug.debugDump(0x5a);
      if (length != 0)
         return (io_ioctlNative(READ, handle, arr, offset, length, timeout,
                                !suspend));
      else
         return 0;
   }

   /**
    * Performs a generic ioctl method.  The driver is responsible for the
    * format of the data passed to it.
    *
    *
    * @param handle
    * @param arr
    * @param length
    * @param timeout
    *
    * @return
    *
    * @see
    */
   protected static int ioctl (int handle, byte[] arr, int offset,
                               int length, int timeout)
   {
      return (io_ioctlNative(IOCTL, handle, arr, offset, length, timeout,
                             true));
   }

   /**
    * returns the number of bytes available in a driver.
    *
    *
    * @param handle
    *
    * @return
    *
    * @see
    */
   protected static int available (int handle)
   {
      return (io_availableNative(handle));
   }
}
