
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.comm;



import java.io.*;


/**
 * An Serial output stream is an output stream for writing data to a serial port.
 */
public class SerialOutputStream
   extends java.io.OutputStream
{
   InternalSerialPort SerialPortObject = null;

   /**
    * Constructor declaration
    *
    * @see
    */
   private SerialOutputStream ()
   {
   }

   /**
    * Creates a new data output stream to write data to the serial port.
    * @param sp the serial port to use for communications.
    *
    */
   public SerialOutputStream (InternalSerialPort sp)
   {
      SerialPortObject = sp;
   }

   /**
    * Opens a serial output stream for writing.
    *
    * @exception  IOException  if an I/O error occurs.
    */
   private void open ()
      throws IOException
   {
      SerialPortObject.open();
   }

   /**
    * Writes the given character to the serial port.  This method blocks until
    * the byte has actually be sent.
    *
    * @param ch the byte to send.
    *
    * @throws IOException
    *
    * @see
    */
   public synchronized void write (int ch)
      throws IOException
   {
      SerialPortObject.write(ch);
   }

   /**
    * Sends the given array out the serial port.  This method blocks until all bytes
    * have been sent.
    *
    * @param barr the array to send.
    * @param offset the offset into the array to begin sending.
    * @param len the number of bytes to send.
    *
    * @throws IOException
    *
    * @see
    */
   public synchronized void write (byte[] barr, int offset, int len)
      throws IOException
   {
      SerialPortObject.write(barr, offset, len);
   }

   /**
    * Closes this output stream and releases any system resources
    * associated with this stream.
    *
    * @exception  IOException  if an I/O error occurs.
    */
   public void close ()
      throws IOException
   {
      SerialPortObject.close();
   }
}
