
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.io;



import com.dalsemi.system.ArrayUtils;
import java.io.CharConversionException;


/**
 *
 */
public class CharToByteUTF8
   extends CharToByteConverter
{

   /**
    *
    */
   public int getMaxByteCount (char[] forThis, int start, int end)
   {
      int utfLength = 0;

      for (int i = start; i < end; i++)
      {
         int ch = forThis [i];

         if (ch < 0x80)
         {
            utfLength++;
         }
         else if (ch < 0x0800)
         {
            utfLength += 2;
         }
         else if (ch < 0x010000)
         {
            utfLength += 3;
         }
      }

      return utfLength;
   }

   /**
    *
    */
   public int convert (char[] src, int srcStart, int srcEnd, byte[] dst,
                       int dstStart, int dstEnd)
      throws CharConversionException
   {
      int pos = dstStart;

      for (int i = srcStart; i < srcEnd; i++)
      {
         int ch = src [i];

         if (ch < 0)
         {
            throw new CharConversionException("<0");
         }
         else if (ch < 0x80)
         {
            dst [pos++] = ( byte ) ch;
         }
         else if (ch < 0x0800)
         {
            dst [pos++] = ( byte ) (0xC0 | ((ch >> 6) & 0x1F));
            dst [pos++] = ( byte ) (0x80 | (ch & 0x3F));
         }
         else
         {
            dst [pos++] = ( byte ) (0xE0 | ((ch >> 12) & 0x0F));
            dst [pos++] = ( byte ) (0x80 | ((ch >> 6) & 0x3F));
            dst [pos++] = ( byte ) (0x80 | (ch & 0x3F));
         }
      }

      return pos - dstStart;
   }

   /**
    *
    */
   public int flush (byte[] buff, int start, int length)
   {
      return 0;
   }

   /**
    *
    */
   public String getName ()
   {
      return "UTF8";
   }
}
