
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

/**
 * BasicClient.java
 * KLA
 *
 * A basic network client with a control socket and a data socket.
 *
 */
package com.dalsemi.protocol;



import java.net.*;
import java.io.*;


public class BasicClient
{
   protected Socket control       = null;
   protected Socket data          = null;
   protected String controlServer = null;
   protected String dataServer    = null;
   protected int    dataPort      = 20;
   protected int    controlPort;

   // used to be 500, but when opening ftp servers, it took longer
   // than that, so I bumped it to 2000.  This is reset by the client
   // extending BasicClient when needed.
   //      protected int timeout = 2000;
   protected InputStream    cin                    = null;   //control streams
   protected OutputStream   cout                   = null;
   protected InputStream    din                    = null;   //data streams
   protected OutputStream   dout                   = null;
   private static final int DEFAULT_SOCKET_TIMEOUT = 10000;

   /**
    * Constructor BasicClient
    *
    *
    * @param SERVER
    * @param PORT
    *
    * @throws IOException
    *
    */
   public BasicClient (String SERVER, int PORT)
      throws IOException
   {
      control = new Socket(SERVER, PORT);

      try
      {
         control.setSoTimeout(DEFAULT_SOCKET_TIMEOUT);

         cin           = new BufferedInputStream(control.getInputStream());
         cout          = control.getOutputStream();
         controlPort   = PORT;
         controlServer = new String(SERVER);
      }
      catch (IOException e)
      {

         // Make sure the socket gets closed on exception.
         control.close();

         throw e;
      }
   }

   /**
    * Method setTimeout
    *
    *
    * @param x
    *
    */
   public void setTimeout (int x)
   {
      try
      {
         control.setSoTimeout(x);
      }
      catch (SocketException e)
      {

         // DRAIN!
      }
   }

   /**
    * Method getTimeout
    *
    *
    * @return
    *
    */
   public int getTimeout ()
   {
      try
      {
         return control.getSoTimeout();
      }
      catch (SocketException e)
      {
         return -1;
      }
   }

   /*
    * It is up to the caller to put on any required \r\n or whatever
    * IF you only want to read data from this port, call this funtion
    * with a null or "" parameter
    */

   /**
    * Method command
    *
    *
    * @param command
    * @param wait
    *
    * @return
    *
    * @throws IOException
    *
    */
   public InputStream command (String command, boolean wait)
      throws IOException
   {
      if (command != null)
         if (command.length() > 0)
         {
            cout.write(command.getBytes());   //write the command
            cout.flush();                     //force it out
         }

      return cin;
   }

   /**
    * Method command
    *
    *
    * @param cmd
    *
    * @return
    *
    * @throws IOException
    *
    */
   public InputStream command (String cmd)
      throws IOException
   {
      return command(cmd, true);
   }

   protected InputStream readDataStream ()
      throws IOException
   {
      if (data == null)
         throw new IOException("Data Socket not created!");

      return din;
   }

   /**
    * Method setupDataSocket
    *
    *
    * @param Dserver
    * @param Dport
    *
    * @throws IOException
    *
    */
   public void setupDataSocket (String Dserver, int Dport)
      throws IOException
   {
      data = new Socket(Dserver, Dport);

      try
      {
         din        = data.getInputStream();
         dout       = data.getOutputStream();
         dataPort   = Dport;
         dataServer = new String(Dserver);
      }
      catch (IOException e)
      {

         // Make sure the socket gets closed on exception.
         control.close();

         throw e;
      }
   }
}
