
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

/*
 * HeaderManager.java
 * A simple class to handle protocol headers.
 *
 * @author KLA
 *
 */
package com.dalsemi.protocol;



public class HeaderManager
{
   int      count  = 0;   //count the number of fields we have
   int      quanta = 5;   //the number of indices we jump each time
   String[] keys;
   String[] fields;

   /**
    * Constructor HeaderManager
    *
    *
    */
   public HeaderManager ()
   {
      keys   = new String [quanta];
      fields = new String [quanta];
   }

   /**
    * Method count
    *
    *
    * @return
    *
    */
   public int count ()
   {
      return count;
   }

   synchronized void grow ()
   {
      String[] temp1 = new String [keys.length + quanta];
      String[] temp2 = new String [fields.length + quanta];

/*      for (int i = 0; i < keys.length; i++)
      {
         temp1 [i] = keys [i];
         temp2 [i] = fields [i];
      }*/
      System.arraycopy(keys,0,temp1,0,keys.length);
      System.arraycopy(fields,0,temp2,0,fields.length);
      

      keys   = temp1;
      fields = temp2;
   }

   /**
    * Method add
    *
    *
    * @param key
    * @param field
    *
    */
   public synchronized void add (String key, String field)
   {
      if (count == keys.length)
         grow();

      keys [count]   = key;
      fields [count] = field;

      count++;
   }

   /**
    * Method get
    *
    *
    * @param key
    *
    * @return
    *
    */
   public synchronized String get (String key)
   {
      if (key == null)
         return null;

      for (int i = 0; i < count; i++)
         if (key.equals(keys [i]))
            return fields [i];

      return null;
   }

   /**
    * Method get
    *
    *
    * @param index
    *
    * @return
    *
    */
   public synchronized String get (int index)
   {
      if (index >= count)
         return null;

      return fields [index];
   }

   /**
    * Method getKey
    *
    *
    * @param index
    *
    * @return
    *
    */
   public synchronized String getKey (int index)
   {
      if (index >= count)
         return null;

      return keys [index];
   }

   /**
    * Method setIfNotSet
    *
    *
    * @param key
    * @param field
    *
    */
   public synchronized void setIfNotSet (String key, String field)
   {
      if (get(key) == null)
      {
         add(key, field);
      }
   }
}
