
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.protocol.file;



import java.io.*;
import java.net.*;
import com.dalsemi.protocol.*;


/**
 * Connection.java
 * A URLConnection class for accessing the local file system.
 *
 * @author KLA
 */
public class Connection
   extends URLConnection
{
   String        path;
   boolean       connected = false;
   File          file      = null;
   InputStream   fin       = null;
   HeaderManager hm;

   /**
    * Constructor Connection
    *
    *
    * @param url
    *
    */
   public Connection (URL url)
   {
      super(url);

      path = com.dalsemi.protocol.URLDecoder.decode(url.getFile());
      hm   = new HeaderManager();

      setFileNameMap(new DefaultFileNameMap());

      String type = guessContentTypeFromName(path);

      if (type != null)
         hm.add("content-type", type);
   }

   /**
    * Method connect
    *
    *
    * @throws IOException
    *
    */
   public void connect ()
      throws IOException
   {
      file      = new File(path);
      fin       = new FileInputStream(file);
      connected = true;

      hm.add("content-length", String.valueOf(file.length()));
   }

   /**
    * Method getInputStream
    *
    *
    * @return
    *
    * @throws IOException
    *
    */
   public InputStream getInputStream ()
      throws IOException
   {
      return ( InputStream ) getContent();
   }

   /**
    * Method getContent
    *
    *
    * @return
    *
    * @throws IOException
    *
    */
   public Object getContent ()
      throws IOException
   {
      if (!connected)
         connect();

      return fin;
   }

   /**
    * Method getHeaderField
    *
    *
    * @param s
    *
    * @return
    *
    */
   public String getHeaderField (String s)
   {
      return hm.get(s);
   }

   /**
    * Method getHeaderFieldKey
    *
    *
    * @param n
    *
    * @return
    *
    */
   public String getHeaderFieldKey (int n)
   {
      return hm.getKey(n);
   }

   /**
    * Method getHeaderField
    *
    *
    * @param n
    *
    * @return
    *
    */
   public String getHeaderField (int n)
   {
      return hm.get(n);
   }
}
