
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

/*
 * Handler.java
 * KLA
 * Extension of the URLStreamHandler class to support ftp protocol.
 *
 * Format for an ftp URL
 * ftp://user:password@host/file
 *
 * NOTE THAT AN FTP REQUEST THROUGH A PROXY SERVER CAN BE DEALT WITH
 * THROUGH HTTP
 *
 *
 *
 */
package com.dalsemi.protocol.ftp;



import java.net.*;
import java.io.*;
import java.util.*;
import com.dalsemi.protocol.http.*;


public class Handler
   extends URLStreamHandler
{
   URL    url;
   String host = null;
   int    port = 0;
   String file = null;
   String ref  = null;

   protected URLConnection openConnection (URL u)
      throws IOException
   {
      url = u;

      String proxyServer = com.dalsemi.tininet.TININet.getProxyServer();
      int    proxyPort   = com.dalsemi.tininet.TININet.getProxyPort();

      if ((proxyServer == null) || (proxyServer.length() == 0))
         return new com.dalsemi.protocol.ftp.Connection(url);
      else
         return new com.dalsemi.protocol.http.Connection(url, proxyServer,
                 proxyPort);
   }

   protected void parseURL (URL url, String spec, int start, int limit)
   {
      host = "";
      port = 0;
      file = "/";
      ref  = null;

      StringTokenizer st = new StringTokenizer(spec.substring(start + 2),
                                               "/");

      parseScheme(st.nextToken());

      if (st.hasMoreTokens())
      {

         //this has a path and a host
         parsePath(spec.substring(spec.indexOf("/", start + 2)));
      }

      setURL(url, "ftp", host, port, file, ref);
   }

   void parseScheme (String scheme)
   {
      int colon = scheme.lastIndexOf(( int ) ':');
      int at    = scheme.lastIndexOf(( int ) '@');
      int max   = 2;

      if (colon > at)
         max = 3;

      //if the number of tokens when we split up by ':' is == max,
      //then we need to preserve the first one since it is a
      //password/username thing
      StringTokenizer st    = new StringTokenizer(scheme, ":");
      int             count = st.countTokens();

      if (count == max)
         host = new String(st.nextToken() + ":" + st.nextToken());
      else
         host = new String(st.nextToken());

      //max==3 means that we have a port declaration
      if (max == 3)
         port = Integer.parseInt(st.nextToken());
      else
         port = 21;
   }

   void parsePath (String path)
   {
      StringTokenizer st = new StringTokenizer(path, ";");

      file = st.nextToken();

      if (st.hasMoreTokens())
      {   //then this path has a ftp-type declaration
         ref = st.nextToken();
      }
   }
}
