
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.protocol.http;



import java.io.*;
import java.net.*;
import java.util.*;
import com.dalsemi.protocol.*;


public class Connection
   extends HttpURLConnection
{
   String           host;
   int              port;
   String           file;
   String           ref;
   String           proxyHost = null;
   int              proxyPort = 0;
   URL              url;
   HeaderManager    hm;
   HeaderManager    reqhm;
   InputStream      httpin  = null;
   HTTPOutputStream httpout = null;
   HTTPClient       http;       //the object that will actually perform

   //the HTTP protocol for us
   boolean connected = false;   //tells us we need to call the connect()

   //method if this has not yet happened
   boolean doneInput = false;   //tells us if we have yet to do any input, so

   //no more output is allowed

   /**
    * Constructor Connection
    *
    *
    * @param url
    *
    */
   public Connection (URL url)
   {
      super(url);

      setFileNameMap(new DefaultFileNameMap());

      hm    = new HeaderManager();
      reqhm = new HeaderManager();
      port  = url.getPort();

      if (port < 0)
         port = 80;

      file     = url.getFile();
      ref      = url.getRef();
      host     = url.getHost();
      this.url = url;
   }

   /**
    * Constructor Connection
    *
    *
    * @param url
    * @param proxyS
    * @param proxyP
    *
    */
   public Connection (URL url, String proxyS, int proxyP)
   {
      this(url);

      proxyHost = proxyS;
      proxyPort = proxyP;
   }

   /**
    * Method connect
    *
    *
    * @throws IOException
    *
    */
   public void connect ()
      throws IOException
   {

      //here's where i need to set up a session and all
      //set up the Sockets, Streams
      if (proxyHost == null)
         http = new HTTPClient(host, port, hm);
      else
         http = new HTTPClient(proxyHost, proxyPort, hm);

      connected = true;
   }

   /**
    * Method getInputStream
    *
    *
    * @return
    *
    * @throws IOException
    *
    */
   public InputStream getInputStream ()
      throws IOException
   {
      if (!doInput)
         throw new IOException("First call setDoInput(true)");

      doneInput = true;

      if (httpin == null)
         return ( InputStream ) getContent();
      else
      {
         if (httpout != null)
         {
            httpout = null;

            return (httpin = http.issueCommand(null));
         }

         return httpin;   //(httpin=http.issueCommand(null));
      }
   }

   /**
    * Method getOutputStream
    *
    *
    * @return
    *
    * @throws IOException
    *
    */
   public OutputStream getOutputStream ()
      throws IOException
   {
      if (!doOutput)
         throw new IOException("First call setDoOutput(true)");

      if (doneInput)
         throw new IOException("Cannot output after input has been done");

      if (!connected)
         connect();

      if (httpout != null)
         return httpout;

      if ((method.equals("GET")) || (method.equals("HEAD")))
         method = "POST";

      if (usingProxy())
         http.issueCommand(method + " " + (url.toString()) + " HTTP/1.0\r\n");
      else
         http.issueCommand(method + " " + file + " HTTP/1.0\r\n");

      http.setRequestHeaderManager(reqhm);
         
      httpout = http.getOutputStream();
      httpin  = http.getInputStream();

      return httpout;
   }

   /**
    * Method getContent
    *
    *
    * @return
    *
    * @throws IOException
    *
    */
   public Object getContent ()
      throws IOException
   {
      doneInput = true;

      if (httpout != null)
      {
         httpout = null;

         //need to get the headers out of the stream
         http.issueCommand(null);
      }

      if (httpin != null)
         return httpin;

      //returns an InputStream
      if (!connected)
         connect();

      reqhm.setIfNotSet("Host", url.getHost());
      http.setRequestHeaderManager(reqhm);

      if (usingProxy())
         httpin = http.issueCommand(method + " " + (url.toString())
                                    + " HTTP/1.0\r\n");
      else
         httpin = http.issueCommand(method + " " + file + " HTTP/1.0\r\n");

      if ((getResponseCode() > 299) && (getResponseCode() < 400))
      {
         if (getFollowRedirects())
         {                          //follow the link in Location
            String newurl = hm.get("Location");

            if (newurl != null)
            {
               URL u = new URL(newurl);

               this.url     = u;
               responseCode = -1;   //so i don't keep re-sending a code
               hm           = new HeaderManager();
               reqhm        = new HeaderManager();

               http.clearHeaderManagers();

               port = u.getPort();

               if (port < 0)
                  port = 80;

               file = u.getFile();
               ref  = u.getRef();
               host = u.getHost();

               if (!http.reConnect())
                  throw new IOException(
                     "Failed to reconnect to server on redirect");

               httpin = null;
               httpin = ( InputStream ) getContent();
               hm     = http.getHeaderManager();

               return httpin;
            }
         }
      }

      return httpin;
   }

   /**
    * Method getHeaderField
    *
    *
    * @param s
    *
    * @return
    *
    */
   public String getHeaderField (String s)
   {
      try
      {
         getContent();
      }
      catch (Exception e){}

      return hm.get(s);
   }

   /**
    * Method getHeaderFieldKey
    *
    *
    * @param n
    *
    * @return
    *
    */
   public String getHeaderFieldKey (int n)
   {
      try
      {
         getContent();
      }
      catch (Exception e){}

      return hm.getKey(n);
   }

   /**
    * Method getHeaderField
    *
    *
    * @param n
    *
    * @return
    *
    */
   public String getHeaderField (int n)
   {
      try
      {
         getContent();
      }
      catch (Exception e){}

      return hm.get(n);
   }

   /**
    * Method disconnect
    *
    *
    */
   public void disconnect ()
   {
      try
      {
         httpin.close();
      }
      catch (IOException ioe)
      {

         //drain
      }
   }

   /* Indicates if the connection is going through a proxy. */

   /**
    * Method usingProxy
    *
    *
    * @return
    *
    */
   public boolean usingProxy ()
   {
      return (proxyHost != null);
   }

   /**
    * Method setRequestProperty
    *
    *
    * @param key
    * @param value
    *
    */
   public void setRequestProperty (String key, String value)
   {
      if (connected)
         throw new IllegalAccessError("Already connected");

      reqhm.setIfNotSet(key, value);
   }

   /**
    * Method getRequestProperty
    *
    *
    * @param key
    *
    * @return
    *
    */
   public String getRequestProperty (String key)
   {
      return reqhm.get(key);
   }
}
