
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.protocol.http;



import java.io.*;
import java.util.Vector;
import com.dalsemi.protocol.HeaderManager;

public class HTTPOutputStream
   extends OutputStream
{

   //this class needs to buffer up everything, count its own length
   //and output its length first as a "Content-length: xxxx\r\n\r\n" string
   //all will be sent out on the flush() or close() command
   ByteArrayOutputStream baos = null;
   OutputStream out          = null;
   boolean      finished = false;
   HeaderManager hm = null;

   /**
    * Constructor HTTPOutputStream
    * This one accepts a header manager so we can output our headers! DOH!
    *
    * @param o
    *
    */
   public HTTPOutputStream (OutputStream o, HeaderManager header)
   {
      out          = o;
      baos = new ByteArrayOutputStream(256);
      hm = header;
   }
   
   //for backwards compatibility
   public HTTPOutputStream (OutputStream o)
   {
      this(o,null);
   }
   

   /**
    * Method write
    *
    *
    * @param x
    *
    * @throws IOException
    *
    */
   public void write (int x)
      throws IOException
   {
      if (finished)
         throw new IOException("Stream has been closed: cannot output");

      synchronized (this)
      {
         baos.write(x);
      }
   }

   /**
    * Method write
    *
    *
    * @param b
    *
    * @throws IOException
    *
    */
   public void write (byte[] b)
      throws IOException
   {
      if (finished)
         throw new IOException("Stream has been closed: cannot output");

      synchronized (this)
      {
         baos.write(b,0,b.length);
      }
   }

   /**
    * Method write
    *
    *
    * @param b
    * @param index
    * @param offset
    *
    * @throws IOException
    *
    */
   public void write (byte[] b, int index, int offset)
      throws IOException
   {
      if (finished)
         throw new IOException("Stream has been closed: cannot output");

      synchronized (this)
      {
         baos.write(b,index,offset);
      }
   }

   /**
    * Method flush
    *
    *
    * @throws IOException
    *
    */
   public void flush ()
      throws IOException
   {
      if (finished)
      {
         return;
      }

      synchronized (this)
      {
          finished = true;
      }

      //DOH! DON'T FORGET THE REQUEST HEADER!!!
      if (hm!=null)
      {
        for (int i=0;i<hm.count();i++)
            out.write((hm.getKey(i)+": "+hm.get(i)+"\r\n").getBytes());
      }

      out.write(("Content-length: " + Integer.toString(baos.size())
                 + "\r\n\r\n").getBytes());


      out.write(baos.toByteArray());
   }

   /**
    * Method close
    *
    *
    * @throws IOException
    *
    */
   public void close ()
      throws IOException
   {
      flush();
   }
}
