
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.shell.server.serial;



import java.io.*;
import java.net.*;
import com.dalsemi.shell.server.*;


/**
 * This is an input stream to be used by a SerialSession.
 *
 * @author Stephen Hess, Lorne Smith
 * @version 1.0
 */
class SerialInputStream
   extends SystemInputStream
{
   int thisLine = 0;

   /**
    * Initialize the stream.
    *
    * @param  in  Underlying InputStream to use.
    * @param  out PrintStream to echo characters
    */
   public SerialInputStream (InputStream in, PrintStream out)
   {
      super(in, out);
   }

   /**
    * Read the next character from the stream.
    *
    * @return  the next character from the stream, or
    * <code>-1</code> if the end of the stream has been
    * reached.
    *
    * @throws IOException if an IO problem occurs during
    * the read.
    */
   protected int rawRead ()
      throws IOException
   {
      int r = root.read();

      while ((session != null) && ((r == 0x1b) &&!session.inCommand()))
      {
         String newCommand;

         root.read();          //Read the 4f;

         r = root.read();

         //Up Arrow or Down Arrow
         if ((r == 0x41) || (r == 0x42))
         {
            if (r == 0x41)
               newCommand = session.stepUpHistory();
            else
               newCommand = session.stepDownHistory();

            if (newCommand == null)
            {
               r = root.read();

               continue;
            }
            byte[] command_bytes = newCommand.getBytes();
            //Clear what was in the buffer
            startPos = 0;
            endPos   = 0;

            //Now insert the new command into the buffer.
            increaseBuffer(command_bytes.length);
            System.arraycopy(command_bytes, 0, buffer, 0, command_bytes.length);
            endPos = command_bytes.length;
            

            if (ECHO)
            {
               for (int k = 0; k < thisLine; k++)
               {
                  out.write(ERASE);
               }

               out.write(command_bytes);
            }

            thisLine = endPos;
            r        = root.read();
         }
         else if (r == 0x44)   // the left arrow==bkspace
         {
            r = 0x08;

            break;
         }
         else if (r == 0x43)   // the right arrow=normal space
         {
            r = 0x20;

            break;
         }
      }

      if (r == -1)
         return -1;   // EOL

      if (ECHO || (r == '\n') || (r == '\r'))
      {
         if ((r != 0x08) && (r != 0x0FF))
         {
            thisLine++;

            out.write(r);
         }
         else if (r == 0x08)
         {
            if (thisLine > 0)
            {
               thisLine--;

               if (ECHO)
                  out.write(ERASE);
            }
         }
      }

      if (r == '\r')
      {
         thisLine = 0;

         out.write('\n');
      }

      return r;
   }
}
