/****************************************************************************
*
*  Copyright (C) 1999,2000 Dallas Semiconductor Corporation. 
*  All rights Reserved. Printed in U.S.A.
*  This software is protected by copyright laws of
*  the United States and of foreign countries.
*  This material may also be protected by patent laws of the United States 
*  and of foreign countries.
*  This software is furnished under a license agreement and/or a
*  nondisclosure agreement and may only be used or copied in accordance
*  with the terms of those agreements.
*  The mere transfer of this software does not imply any licenses
*  of trade secrets, proprietary technology, copyrights, patents,
*  trademarks, maskwork rights, or any other form of intellectual
*  property whatsoever. Dallas Semiconductor retains all ownership rights.
*
*     Module Name: DiffCommand.java
*/
package com.dalsemi.slush.command;

import java.io.*;
import java.util.*;
import com.dalsemi.slush.*;
import com.dalsemi.system.*;
import com.dalsemi.shell.server.*;

/**
 * Compare two binary files, byte for byte.
 *
 * @author Steve Curry
 * @version 1.0
 */

public class DiffCommand implements SlushCommand
{
    /**
     * Returns a string describing the usage of the command and its arguments if any.
     *
     *
     * @return The usage statement for this command.
     */
    public String getUsageString()
    {
        return "diff FILE1 FILE2\r\n\r\nCompare byte for byte FILE1 to FILE2." +
               "\r\nWhen a difference is found, the offset and the differing" +
               "\r\nbytes are printed.";
    }
    
    /**
     * Method called when this command is executed from the command prompt.
     *
     *
     * @param  in   The <code>InputStream</code> this command will use to receive input.
     * @param  out  The <code>PrintStream</code> should use to report any non-critical messages.
     * @param  err  The <code>PrintStream</code> should use to report any critical messages.
     * @param  args The command line arguments passed to this command.
     * @param  env  The current environment variables for this session.
     * @throws Exception
     */
    public void execute(SystemInputStream in, SystemPrintStream out, SystemPrintStream err, 
                        String[] args, Hashtable env) throws Exception
    {
        if (args.length == 2)
        {
            File f = new File(args[0]);
            try
            {
                BufferedReader brf = new BufferedReader(new InputStreamReader(new FileInputStream(f)));
                
                File g = new File(args[1]);
                try
                {
                    BufferedReader brg = new BufferedReader(new InputStreamReader(new FileInputStream(g)));
                    long n;
                    if ((n = f.length()) == g.length())
                    {
                        boolean good = true;
                        try
                        {
                            for (long i = 0; i < n; i++)
                            {
                                int cf = brf.read();
                                int cg = brg.read();
                                if (cf != cg)
                                {
                                    out.println(Long.toString(i, 16) + ": " +
                                                Integer.toString(cf, 16) + " " +
                                                Integer.toString(cg, 16));
                                    good = false;
                                }
                            }
                            if (good)
                                out.println("Files are identical.");
                            else
                                out.println("Files differ.");
                        }
                        catch (IOException e)
                        {
                            out.println(e);
                        }
                    }
                    else
                        out.println("Files are not the same length");
                    
                    try
                    {
                        brg.close();
                    }
                    catch (IOException e)
                    {
                        //DRAIN
                    }
                }
                catch (FileNotFoundException e)
                {
                    out.println(args[1] + " does not exist.");
                }
                
                try
                {
                    brf.close();
                }
                catch (IOException e)
                {
                    //DRAIN
                }
            }
            catch (FileNotFoundException e)
            {
                out.println(args[0] + " does not exist");
            }
        }
        else
            out.println(getUsageString());
    }
}