/****************************************************************************
*
*  Copyright (C) 1999,2000 Dallas Semiconductor Corporation. 
*  All rights Reserved. Printed in U.S.A.
*  This software is protected by copyright laws of
*  the United States and of foreign countries.
*  This material may also be protected by patent laws of the United States 
*  and of foreign countries.
*  This software is furnished under a license agreement and/or a
*  nondisclosure agreement and may only be used or copied in accordance
*  with the terms of those agreements.
*  The mere transfer of this software does not imply any licenses
*  of trade secrets, proprietary technology, copyrights, patents,
*  trademarks, maskwork rights, or any other form of intellectual
*  property whatsoever. Dallas Semiconductor retains all ownership rights.
*
*     Module Name: MemoryReporter.java
*/
package com.dalsemi.slush.command;

import java.net.*;
import java.io.*;


public class MemoryReporter
  extends Thread
{
  private static final String NAME = "MemoryReporter";
  private ServerSocket  serverSocket;
  private boolean       testInProgress;
  private long          interval      = 1000;
  private int           portNumber    = 4401;
  int                   freeRAM       = 0;
  byte[]                buffer        = new byte[4];
  PrintStream          sysout;
  MemoryReporterListener listener;

  Socket        socket = null;
  InputStream   in     = null;
  OutputStream  out    = null;

  /** start a thread that will begin reporting the memory
    * usage via TCP on port <code>portNumber</code> at
    * <code>interval</code> intervals.
    */
  public void startTest(long interval, int portNumber, PrintStream out,
                        MemoryReporterListener listener)
  {
    try
    {
      sysout = out;
      this.listener = listener;
      this.portNumber = portNumber;
      serverSocket    = new ServerSocket(portNumber);
      testInProgress  = true;
      this.interval   = interval;
      start();
    }
    catch(Throwable t)
    {
      sysout.println(toString() + " - startTest failed: " + t.toString());
      if(listener != null)
      {
        listener.reporterStopped(); //DEBUG -- MultipleInterfaceBug??
      }
    }
  }
  /** start a thread that will begin reporting the memory
    * usage via TCP on the default port at
    * <code>interval</code> intervals.
    */
  public void startTest(int interval)
  {
    startTest(interval, portNumber, System.out, null);
  }
  /** start a thread that will begin reporting the memory
    * usage via TCP on the default port at default
    * intervals.
    */
  public void startTest()
  {
    startTest(interval, portNumber, System.out, null);
  }
  /**
    */
  public void run()
  {
//    System.out.println(toString() + " - starting report server");
    try
    {
      socket  = serverSocket.accept();
      in      = socket.getInputStream();
      out     = socket.getOutputStream();

      while(testInProgress)
      {
        freeRAM = com.dalsemi.system.TINIOS.getFreeRAM();
        buffer[0] = (byte)(freeRAM & 0x0ff);
        buffer[1] = (byte)((freeRAM >>> 8)  & 0x0ff);
        buffer[2] = (byte)((freeRAM >>> 16) & 0x0ff);
        buffer[3] = (byte)((freeRAM >>> 24) & 0x0ff);
        try
        {
            out.write(buffer);
        }
        catch(IOException ioe)
        {
            testInProgress = false;
        }
        Thread.sleep(interval);
      }
    }
    catch(Throwable t)
    {
      sysout.println(toString() + " - Aborted report server: " + t.toString());
    }
    
    sysout.print("Stopping current reporter...");

    try
    {
      in.close();
      out.close();
      socket.close();
      serverSocket.close();
    }
    catch(IOException io)
    {
      sysout.println(toString() + " - Unable to close socket: " + io.toString());
    }
    
    if(listener != null)
    {
        listener.reporterStopped();//DEBUG -- MultipleInterfaceBug??
    }
    
    sysout.println("[Done]");
  }
  /** stop the report server
    */
  public void stopTest()
  {
    sysout.print("Stopping current reporter...");
    try
    {
      if(in != null)
        in.close();
      if(out != null)
        out.close();
      if(socket != null)
        socket.close();
      if(serverSocket != null)
        serverSocket.close();
    }
    catch(IOException io)
    {
      sysout.println(toString() + " - Unable to close socket: " + io.toString());
    }

    if(listener != null)
    {
        listener.reporterStopped();//DEBUG -- MultipleInterfaceBug??
    }

    sysout.println("[Done]");
  }
  /** return our name
    */
  public String toString()
  {
    return NAME;
  }
}