/****************************************************************************
*
*  Copyright (C) 1999,2000 Dallas Semiconductor Corporation. 
*  All rights Reserved. Printed in U.S.A.
*  This software is protected by copyright laws of
*  the United States and of foreign countries.
*  This material may also be protected by patent laws of the United States 
*  and of foreign countries.
*  This software is furnished under a license agreement and/or a
*  nondisclosure agreement and may only be used or copied in accordance
*  with the terms of those agreements.
*  The mere transfer of this software does not imply any licenses
*  of trade secrets, proprietary technology, copyrights, patents,
*  trademarks, maskwork rights, or any other form of intellectual
*  property whatsoever. Dallas Semiconductor retains all ownership rights.
*
*     Module Name: SledCommand.java
*/
package com.dalsemi.slush.command;

import com.dalsemi.shell.server.*;
import com.dalsemi.slush.command.sled.*;
import java.util.*;
import java.io.*;
import com.dalsemi.slush.*;

/**
 * Basic vi like text editor.  I stress the "like" portion of that last 
 * sentence.  See the documentation in <code>Sled</code> for a list of
 * supported features.
 *
 * @author  Stephen Hess
 * @version 1.0
 */
public class SledCommand implements SlushCommand
{
    /**
     * Returns a string describing the usage of the command and its arguments if any.
     *
     *
     * @return The usage statement for this command.
     */
    public String getUsageString()
    {
        return "sled [file]\r\n\r\nEdit the given textfile.";
    }

    /**
     * Method called when this command is executed from the command prompt.
     *
     *
     * @param  in   The <code>InputStream</code> this command will use to receive input.
     * @param  out  The <code>PrintStream</code> should use to report any non-critical messages.
     * @param  err  The <code>PrintStream</code> should use to report any critical messages.
     * @param  args The command line arguments passed to this command.
     * @param  env  The current environment variables for this session.
     * @throws Exception
     */
    public void execute(SystemInputStream in, SystemPrintStream out, SystemPrintStream err, 
                        String[] args, Hashtable env) throws Exception
    {
        out.println("Starting sled, please wait...");
        in.setEcho(false);
        in.setRawMode(true);

        new Sled(in, out, ((args.length > 0) ? args[0] : null));

        in.setEcho(true);
        //Now, make sure to clear out anything we may have left in the buffer.
        while (in.available() > 0)
        {
            in.read();
        }
        //DEBUG ==> Should I check raw mode first so I don't wreck what
        //was there?
        in.setRawMode(false);
    }
}