/****************************************************************************
*
*  Copyright (C) 1999,2000 Dallas Semiconductor Corporation. 
*  All rights Reserved. Printed in U.S.A.
*  This software is protected by copyright laws of
*  the United States and of foreign countries.
*  This material may also be protected by patent laws of the United States 
*  and of foreign countries.
*  This software is furnished under a license agreement and/or a
*  nondisclosure agreement and may only be used or copied in accordance
*  with the terms of those agreements.
*  The mere transfer of this software does not imply any licenses
*  of trade secrets, proprietary technology, copyrights, patents,
*  trademarks, maskwork rights, or any other form of intellectual
*  property whatsoever. Dallas Semiconductor retains all ownership rights.
*
*     Module Name: SledEvent.java
*/
package com.dalsemi.slush.command.sled;

import java.util.*;

/**
 * This class encapsulates a SledEvent and is used or redo and undo
 * operations.
 *
 * @author Stephen Hess
 * @version 1.0
 */
public class SledEvent
{
    //DEBUG ==> Know is isn't very Java-esque, but I'm not going to
    //use getters and setters here.  I know, I know, thunder and lightening,
    //but I'm more worried about speed at this point than information hiding.
    //Such is life.
    //
    //Event types include:
    //      CHANGING:  i, a, A, cw, dw, D, C
    //      DELETING:  dd, #dd
    //      INSERT  :  o, O
    public static final int DELETING_CHARS   =  0;
    public static final int DELETING_LINES   =  1;
    public static final int DELETING_WORD    =  2;
    public static final int REPLACING_CHAR   =  3;
    public static final int INSERTING        =  4;
    public static final int PASTE            =  5;
    public static final int JOINING_LINES    =  6;
    public static final int INSERTING_FILE   =  7;
    public static final int SEARCH_REPLACE   =  8;
    
    public int    type;
    public Vector changed;
    public int    startLine;
    public int    startCursor;
    public int    miscInfo;
    public char   miscInfoB;
    public Vector misc;     //Used mostly for search/replace line numbers
    public char   command;
    
    public SledEvent(int type, int expectedBuffSize, int startLine,
                     int startCursor, int miscInfo, char miscInfoB,
                     char command)
    {
        this.type = type;
        this.startLine = startLine;
        this.startCursor = startCursor;
        this.miscInfo = miscInfo;
        this.miscInfoB = miscInfoB;
        this.command = command;
        if (expectedBuffSize != -1)
        {
            changed = new Vector(expectedBuffSize, 10);
        }
    }
}