
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.system;



import java.util.*;

/**
 * This interface allows an application to respond when an external interrupt
 * has been triggered. ExternalInterruptEventListeners cannot expect to receive
 * a real-time response to these interrupts nor can they expect to receive a
 * response to all such interrupts if interrupts arrive in close succession or
 * if TINI is CPU bound servicing other threads and/or processes. Real-time
 * response to external interrupts requires a native library to replace the
 * external interrupt service routine.
 *
 * @see ExternalInterrupt
 * @see ExternalInterruptEvent
 * @see ExternalInterruptException
 */
public interface ExternalInterruptEventListener
   extends EventListener
{

   /**
    * Propagates a ExternalInterrupt event.
    */
   public abstract void externalInterruptEvent (ExternalInterruptEvent ev);
}
