
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.tininet.dhcp;



/**
 * DHCPListener interface to accept dhcp status messages
 *
 *
 * @author CM
 */
public interface DHCPListener
{

   /**
    * Invoked when an IP is leased. Calling application should do any setup required here
    *
    *
    */
   public void ipLeased ();

   /**
    * Invoked when the IP was renewed.
    *
    *
    */
   public void ipRenewed ();

   /**
    * Invoked when client loses current IP.
    *
    *
    */
   public void ipLost ();

   /**
    * Invoked when Error occurs in dhcp attempt. Thread will continue execution, attempting
    * to get lease from server until manually stopped by the originator. 
    *
    *
    *
    * @param error status error that occurred during dhcp attempt, possible errors include<p>
    *
    *        "No reply in Init"<br>
    *        "No reply in Selecting"<br>
    *        "No reply in Requesting"<br>
    *        "No reply in Bound"<br>
    *        "No reply in Renewing"<br>
    *        "No reply in Rebinding"<br>
    *        "No reply in Init Reboot"<br>
    *        "No reply in Rebooting"<br>
    *        "Unknown Host Exception"<br>
    *        "IOException"<br>
    *        "Fatal Error"<br>
    *        "Unknown Error"<p>
  
    */
   public void ipError (String error);
}
