/* macro.l */
/* Lexical Analyzer */

/*  
  Copyright (C) 1999, 2000 Dallas Semiconductor Corporation.
  All rights Reserved. Printed in U.S.A.
  This software is protected by copyright laws of
  the United States and of foreign countries.
  This material may also be protected by patent laws of the United States
  and of foreign countries.
  This software is furnished under a license agreement and/or a
  nondisclosure agreement and may only be used or copied in accordance
  with the terms of those agreements.
  The mere transfer of this software does not imply any licenses
  of trade secrets, proprietary technology, copyrights, patents,
  trademarks, maskwork rights, or any other form of intellectual
  property whatsoever. Dallas Semiconductor retains all ownership rights.
*/

/* BUILD INFORMATION:
     FLEX 2.5.4
     GCC 2.91.66 or MSDEVSTUDIO/VC 6
 */

/* LIMITATIONS:
 *   - This is a single pass processor. This means that *ALL* equates
 *     and macros must be defined before they are used.
 *   - Compound equate expressions are *NOT* allowed within macros. 
 *     A sure sign that this rule has been broken is the existance
 *     of the string "(null)" in the output file where the compound equate
 *     was used.
 *   - There are problems with mixed strings and constants in DB
 *     statements. 
 *      Example - DB "Version ", ASCII_MAJOR, ".", ASCII_MINOR, 0
 *     This can be spread over several lines to correct the problem.
 *      Example - DB "Version "
 *                DB ASCII_MAJOR
 *                DB "."
 *                DB ASCII_MINOR, 0
 */

/* TODO:  - fix equates that are bit fields of other equates or numbers
 *          e.g. ACON EQU 09dh
 *               AM0  EQU ACON.0
 *               AM1  equ 09dh.1  
 */

/*
 $Workfile: macro.l $
 $Revision: 7 $
 $Date: 9/11/00 5:33a $
 $Author: Chenot $
 $Modtime: 9/11/00 5:33a $
 $Log: /TINI/390 Assembler/source/macro/macro.l $
* 
* 7     9/11/00 5:33a Chenot
* 
* 5     9/08/00 9:35p Chenot
* 
* 4     9/08/00 12:39p Chenot
* 
* 3     5/16/00 7:32a Chenot
* 
* 2     5/15/00 6:11a Chenot
* 
* 60    5/03/00 3:53p Chenot
* 
* 59    5/03/00 8:17a Chenot
* 
* 58    5/03/00 6:16a Chenot
* 
* 57    3/17/00 6:17a Chenot
* 
* 56    1/06/00 7:26a Tomc
* 
* 55    8/11/99 10:02a Tomc
* 
* 54    8/11/99 10:01a Tomc
* 
* 53    7/29/99 1:35p Tomc
* 
* 52    7/27/99 7:37a Tomc
* 
* 51    7/12/99 1:36p Tomc
* 
* 50    7/01/99 11:36a Tomc
* 
* 49    7/01/99 11:17a Tomc
* 
* 48    7/01/99 10:23a Tomc
* 
* 47    7/01/99 10:08a Tomc
* 
* 46    7/01/99 9:45a Tomc
* 
* 45    6/30/99 1:36p Tomc
* 
* 44    6/30/99 1:29p Tomc
* 
* 43    6/30/99 12:35p Tomc
* 
* 42    5/21/99 11:35a Tomc
* 
* 41    5/21/99 8:22a Tomc
* 
* 40    5/20/99 12:04p Tomc
* 
* 39    5/20/99 11:49a Tomc
* 
* 38    5/20/99 10:13a Tomc
* 
* 37    5/20/99 8:23a Tomc
* 
* 36    5/19/99 1:54p Tomc
* 
* 35    5/19/99 1:27p Tomc
* 
* 34    5/19/99 1:08p Tomc
* 
* 33    5/19/99 8:43a Tomc
* 
* 32    5/18/99 3:21p Tomc
* 
* 31    5/18/99 2:38p Tomc
* 
* 30    5/18/99 11:12a Tomc
* 
* 29    5/18/99 9:26a Tomc
* 
* 28    5/17/99 11:04a Tomc
* 
* 27    5/14/99 3:58p Tomc
* 
* 26    5/14/99 3:16p Tomc
* 
* 25    5/04/99 8:46a Tomc
* 
* 24    4/30/99 12:40p Tomc
* 
* 23    4/30/99 7:58a Tomc
* 
* 22    4/29/99 3:38p Tomc
* 
* 21    4/28/99 3:07p Tomc
* Added provision to allow "?Seg At" and db bytes from include files
* 
* 20    4/23/99 8:15a Tomc
* Works on a version of our source base
* 
* 19    4/22/99 11:44a Tomc
* 
* 18    4/22/99 9:41a Tomc
* Now iterative parser instead of recursive
* 
* 17    4/21/99 3:24p Tomc
* 
* 16    4/21/99 2:28p Tomc
* 
* 15    4/21/99 12:51p Tomc
* parser works
* 
* 14    4/20/99 3:46p Tomc
* 
* 13    4/20/99 9:31a Tomc
* 
* 12    4/20/99 8:07a Tomc
* 
* 11    4/19/99 3:34p Tomc
* 
* 10    4/19/99 10:23a Tomc
* 
* 8     4/16/99 3:08p Tomc
* 
* 7     4/15/99 3:40p Tomc
* 
* 6     4/15/99 9:39a Tomc
* 
* 5     4/15/99 9:12a Tomc
* 
* 4     4/15/99 5:27a Tomc
*/

/*  Definitions and C pass-throughs  */

%{
#ifdef WIN32
#ifdef FLEX_SCANNER
#undef YY_DO_BEFORE_ACTION
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;\
  g_lpTempByte = yy_bp;\
  oldpplineno = pplineno; \
  while(g_lpTempByte < yy_cp) \
  { \
    if(*g_lpTempByte == 10) /* NewLine */ \
    {\
      pplineno++; \
      if(*(g_lpTempByte-1) == 0x0d)\
        *(g_lpTempByte-1) = 10;\
    }\
    g_lpTempByte++; \
  }
#undef unput
#define unput(c)  yyunput( c, yytext_ptr );\
                  if(c == 10)\
                    pplineno--;
#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
    pplineno = oldpplineno; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

int   pplineno = 1,
      oldpplineno = 1;
char  *g_lpTempByte;
#endif/* FLEX_SCANNER */ 
#endif /* WIN32 */
#undef YY_MAIN
/***************************************************************************/
/************************* begin my lex declarations ***********************/

#include "macro.h"

#define MACRO_COMMENT_PREFIX ";Comment Inserted By Macro Preprocessor - "

extern  int   yyerror (char *mesg);
extern  int   yywrap  ();
extern  char  *g_FileName;

void          InitIncludeStack(void);
BOOL          PushIncludeFile(char* p_lpFileName);
BOOL          PopIncludeFile();
const char    *GetCurrentFileName(void);
FILE          *GetRootFile(void);
char          *ExtractIncludeFileName();
int           ResolveNumber(char *p_NumberString);
BOOL          IsParam(char *p_lpString);
char          *IsLocal(char *p_lpString);
int           StrCmp(char *p_S1, char *p_S2);
unsigned int  Hash(unsigned char *p_lpKey);
LPMACRO_LIST  IsMacro(char *p_lpIdentifier);
BOOL          InsertEquate();
LPEQUATE_LIST IsEquate(char *p_lpIdentifier);
BOOL          InsertMacro();
BOOL          InsertReference(LPREFERENCE_LIST p_lpReference);
void          WriteMacro(LPMACRO_LIST p_lpMacro, char *p_lpParams);
void          InitMacros();
void          FreeMacros();

char              g_TempString[260];
char              g_CurrentIdentifier[260];
char              g_TempString2[260];
int               g_LineNumber          = 1;
BOOL              g_DebugOutput         = FALSE;
BOOL              g_KillEnd             = FALSE;
BOOL              g_InMacro             = FALSE;
BOOL              g_InEquate            = FALSE;
BOOL              g_HaveBitDigit        = FALSE;
BOOL              g_InMacroHeader       = FALSE;
BOOL              g_InParamList         = FALSE;
BOOL              g_InLocalList         = FALSE;
BOOL              g_InIncludeFile       = FALSE;
BOOL              g_ParsingMultiStorage = FALSE;
BOOL              g_LastIdInEquateList  = FALSE;
BOOL              g_ProcessingNestedMacro = FALSE;
MACRO             g_CurrentMacro;
EQUATE            g_CurrentEquate;
LPMACRO_LIST      //g_MacroList           = NULL,
                  g_CurMacro;
//LPEQUATE_LIST     g_EquateList          = NULL;
LPREFERENCE_LIST  g_lpTempReference,
                  g_lpHoldReference;
void              *g_lpVoidPtr          = NULL;
char              g_LastOperator[256];
int               g_NumberToResolve;
char              *g_lpTempPtr,
                  *g_lpTempPtr2;
int               g_Index;
int               g_LabelCount          = 0;
char              g_LabelPrefix[256];
FILE              *g_lpOutFile;
fpos_t            g_FilePosition,
                  g_ExpressionPosition;

#define   C_SEG   0
#define   D_SEG   1
#define   X_SEG   2
#define   I_SEG   3
#define   B_SEG   4

unsigned  long  g_SegmentOrg[5] = {0,0,0,0,0};
int             g_CurrSegType   = C_SEG;
char            g_ReservedData[128];

INCLUDE_TABLE g_FileStack;        

#define S_INITIAL INITIAL
                              
/************************* end my lex declarations *************************/
/***************************************************************************/
%}

%x S_MACRO
%x S_PARM_EXP

HexNumber         ("0"[a-fA-F]([a-fA-F]|[0-9])+[Hh])|([0-9]([a-fA-F]|[0-9])*[Hh])
DecNumber         [0-9]{1,8}[Dd]{0,1}
BinNumber         [0-1]{1,24}[Bb]
Number            {HexNumber}|{DecNumber}|{BinNumber}
CharString1       \"[^\"]+\"
CharString2       \'[^\']+\'
CharString        {CharString1}|{CharString2}
WhiteSpace        [ \t]
Macro             [Mm][Aa][Cc][Rr][Oo]
EndMacro          {Separator}*[Ee][Nn][Dd][Mm]
Param             [Pp][Aa][Rr][Aa][Mm]
Local             [Ll][Oo][Cc][Aa][Ll]
Separator         {WhiteSpace}+
Shr               {Separator}[Ss][Hh][Rr]{Separator}
Shl               {Separator}[Ss][Hh][Ll]{Separator}
And               {Separator}[Aa][Nn][Dd]{Separator}
Or                {Separator}[Oo][Rr]{Separator}
Logical           {Shr}|{Shl}|{And}|{Or}
Extern            [Ee][Xx][Tt][Rr][Nn]
Public            [Pp][Uu][Bb][Ll][Ii][Cc]
Org               [Oo][Rr][Gg]
End               [Ee][Nn][Dd]
At                [Aa][Tt]
DB                [Dd][Bb]
DW                [Dd][Ww]
DT                [Dd][Tt]
MultiStorage      ({DB}|{DW}|{DT}){Separator}+
DS                [Dd][Ss]
DBit              [Dd][Bb][Ii][Tt]
Segment           [Ss][Ee][Gg][Mm][Ee][Nn][Tt]
Bit               [Bb][Ii][Tt]
Code              [Cc][Oo][Dd][Ee]
Data              [Dd][Aa][Tt][Aa]
IData             [Ii][Dd][Aa][Tt][Aa]
XData             [Xx][Dd][Aa][Tt][Aa]
SegType           ({Bit}|{Code}|{Data}|{IData}|{XData}){Separator}*
RSeg              [Rr][Ss][Ee][Gg]
CSeg              [Cc][Ss][Ee][Gg]
DSeg              [Dd][Ss][Ee][Gg]
XSeg              [Xx][Ss][Ee][Gg]
ISeg              [Ii][Ss][Ee][Gg]
BSeg              [Bb][Ss][Ee][Gg]
SegSelect         {CSeg}|{DSeg}|{XSeg}|{ISeg}|{BSeg}
SegSelectAt       {SegSelect}{Separator}+{At}{Separator}+{Number}
Name              [Nn][Aa][Mm][Ee]
High              [Hh][Ii][Gg][Hh]
Low               [Ll][Oo][Ww]
Not               [Nn][Oo][Tt]
Arithmetic        "+"|"-"|"*"|"/"
Operator          {Arithmetic}|{Logical}
Letter            [a-zA-Z_?]
SpecialChars      [$]
Digit             [0-9]
BitDigit          "."[0-7]
Equate            {Separator}[Ee][Qq][Uu]{Separator}
ValidIdChars      ({Letter}|{Digit}|{SpecialChars})
Identifier        {Letter}{ValidIdChars}*
Label             ^{Identifier}":"
UnitStorage       ({DS}|{DBit}){Separator}+{Number}
BitStorage        {Identifier}{Separator}+{Bit}{Separator}+{Identifier}{BitDigit}
ValidFileNameChars {Identifier}|"~"|"-" 
FileName          {ValidFileNameChars}+("."{ValidFileNameChars})*
Directory         ({Letter}":"){0,1}(("."){1,2}|{FileName})*("\\"|"/")
Include           "$"[Ii][Nn][Cc][Ll][Uu][Dd][Ee]{WhiteSpace}*"("{WhiteSpace}*{Directory}*{FileName}{WhiteSpace}*")".*
Comma             ","
Comment           ";".*
Expression        (({Number}|{Identifier}){Operator})*({Number}|{Identifier})
ExpressionEx      {Expression}|("("{Expression}")")
DollarAddr        "$"({Arithmetic}{ExpressionEx})*
MultiExpression   {ExpressionEx}
HashMark          "#"
Immediate         {HashMark}{MultiExpression}
Indirect          "@"{Identifier}({Arithmetic}{ExpressionEx})*
NewLine           "\n"|"\r\n" 
Control           ^"$".*
Directive         ^({Extern}|{Public}).*
PassThrough       {Directive}|{Control}
Other             .

%%
";!DEBUG!"         { /* DEBUG */
                    g_DebugOutput = !g_DebugOutput;

                    if(g_DebugOutput)
                      printf("Debug ");
                  }

{Comment}         {	/* Comment */

                    if(g_DebugOutput)
                      printf("Comment[%s] ",yytext);

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);
                  }

{Macro}           { /* Macro */
                    if(g_DebugOutput)
                      printf("Macro ");

                    /* back up before last identifier */
                    fsetpos(g_lpOutFile, &g_FilePosition);

                    g_InMacro = TRUE;
                    g_InMacroHeader = TRUE;
                    memset(&g_CurrentMacro, 0,  sizeof(g_CurrentMacro));
                    g_CurrentMacro.Name       = malloc(strlen(g_CurrentIdentifier)+1);

                    if(!g_CurrentMacro.Name)
                      yyerror("Memory allocation error");

                    g_CurrentMacro.Body       = malloc(yy_current_buffer->yy_n_chars<<3);// 8X is a random guess for size

                    if(!g_CurrentMacro.Body)
                      yyerror("Memory allocation error");
/// may want to make this more robust by figuring out the number of bytes to malloc
                    g_CurrentMacro.Params     = malloc(256);

                    if(!g_CurrentMacro.Params)
                      yyerror("Memory allocation error");

/// may want to make this more robust by figuring out the number of bytes to malloc
                    g_CurrentMacro.Locals     = malloc(256);

                    if(!g_CurrentMacro.Locals)
                      yyerror("Memory allocation error");

                    g_CurrentMacro.Body[0]    = 0;
                    g_CurrentMacro.Params[0]  = 0;
                    g_CurrentMacro.Locals[0]  = 0;
                    strcpy(g_CurrentMacro.Name, g_CurrentIdentifier);
                    sprintf(g_CurrentMacro.Body, "; Begin Macro %s\n",g_CurrentIdentifier);
                    BEGIN S_MACRO;
                    return TOKEN_MACRO;
                  }

{Include}         {	/* Include */
                    if(g_DebugOutput)
                      printf("Include[%s] ",yytext);

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    PushIncludeFile(ExtractIncludeFileName());
                      
                  }

{Directive}       { /* Directive */
                    if(g_DebugOutput)
                      printf("Directive[%s] ",yytext);

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);
                  }

{UnitStorage}.*   { /* UnitStorage */ 
                    if(g_DebugOutput)
                      printf("UnitStorage[%s] ",yytext);
// 05-18-99 late hack                    
// 05-18-99 late hack                    g_lpTempPtr = yytext;
// 05-18-99 late hack                    while(!isspace(*g_lpTempPtr++)); /* move past DS      */
// 05-18-99 late hack                    while(isspace(*g_lpTempPtr++));  /* move to NUMBER    */
// 05-18-99 late hack                    g_lpTempPtr--;
// 05-18-99 late hack
// 05-18-99 late hack                    g_NumberToResolve = ResolveNumber(g_lpTempPtr);
// 05-18-99 late hack
// 05-18-99 late hack                    switch(toupper(yytext[1]))
// 05-18-99 late hack                    {
// 05-18-99 late hack                      case 'S':
// 05-18-99 late hack                        for(g_Index = 0; g_Index < g_NumberToResolve; g_Index++)
// 05-18-99 late hack                        {
// 05-18-99 late hack                          if(g_ReservedData[g_SegmentOrg[D_SEG]])
// 05-18-99 late hack                            yyerror("Overlap detected in Direct Segment");
// 05-18-99 late hack
// 05-18-99 late hack                          g_ReservedData[g_SegmentOrg[D_SEG]++] = 1;
// 05-18-99 late hack                        }
// 05-18-99 late hack                      break;
// 05-18-99 late hack
// 05-18-99 late hack                      default:
// 05-18-99 late hack                      break;
// 05-18-99 late hack                    }

                    if(!g_InIncludeFile)
                      //fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);
                      fprintf(g_lpOutFile, "%s", yytext);

                  }

{Label}{Separator}*{UnitStorage}.* { /* UnitStorage */
 
                    if(g_DebugOutput)
                      printf("UnitStorage[%s] ",yytext);

// 05-18-99 late hack                    g_lpTempPtr = yytext;
// 05-18-99 late hack                    while(!isspace(*g_lpTempPtr++)); /* move past Label   */
// 05-18-99 late hack
// 05-18-99 late hack                    strcpy(g_TempString, yytext);                    
// 05-18-99 late hack                    g_TempString[g_lpTempPtr - yytext - 2] = 0;
// 05-18-99 late hack
// 05-18-99 late hack//                    yylval.sym = looksym(g_TempString);
// 05-18-99 late hack//                    yylval.sym->type  = LABEL;
// 05-18-99 late hack
// 05-18-99 late hack                    while(isspace(*g_lpTempPtr++));  /* move to DS        */
// 05-18-99 late hack                    strcpy(g_CurrentIdentifier, g_lpTempPtr);                    
// 05-18-99 late hack
// 05-18-99 late hack                    while(!isspace(*g_lpTempPtr++)); /* move past DS      */
// 05-18-99 late hack                    while(isspace(*g_lpTempPtr++));  /* move to NUMBER    */
// 05-18-99 late hack                  
// 05-18-99 late hack                    g_lpTempPtr--;
// 05-18-99 late hack
// 05-18-99 late hack                    memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
// 05-18-99 late hack                    g_CurrentEquate.Name = malloc(strlen(g_TempString)+1);
// 05-18-99 late hack                    strcpy(g_CurrentEquate.Name, g_TempString);
// 05-18-99 late hack                    g_CurrentEquate.Value = g_SegmentOrg[D_SEG];
// 05-18-99 late hack                    InsertEquate();
// 05-18-99 late hack
// 05-18-99 late hack                    g_NumberToResolve = ResolveNumber(g_lpTempPtr);
// 05-18-99 late hack
// 05-18-99 late hack                    switch(toupper(*g_CurrentIdentifier))
// 05-18-99 late hack                    {
// 05-18-99 late hack                      case 'S':                      
// 05-18-99 late hack                        for(g_Index = 0; g_Index < g_NumberToResolve; g_Index++)
// 05-18-99 late hack                        {
// 05-18-99 late hack                          if(g_ReservedData[g_SegmentOrg[D_SEG]])
// 05-18-99 late hack                            yyerror("Overlap detected in Direct Segment");
// 05-18-99 late hack
// 05-18-99 late hack                          g_ReservedData[g_SegmentOrg[D_SEG]++] = 1;
// 05-18-99 late hack                        }
// 05-18-99 late hack                      break;
// 05-18-99 late hack
// 05-18-99 late hack                      default:
// 05-18-99 late hack                      break;
// 05-18-99 late hack                    }
// 05-18-99 late hack
// 05-18-99 late hack///
// 05-18-99 late hack///                    g_lpTempPtr = yytext;
// 05-18-99 late hack///                    while(!isspace(*g_lpTempPtr++)); /* move past Label   */
// 05-18-99 late hack///
// 05-18-99 late hack///                    strcpy(g_CurrentIdentifier, yytext);
// 05-18-99 late hack///                    g_CurrentIdentifier[g_lpTempPtr - yytext - 2] = 0;
// 05-18-99 late hack///
// 05-18-99 late hack///                    while(isspace(*g_lpTempPtr++));  /* move to DS        */
// 05-18-99 late hack///
// 05-18-99 late hack///                    while(!isspace(*g_lpTempPtr++)); /* move past DS      */
// 05-18-99 late hack///                    while(isspace(*g_lpTempPtr++));  /* move to NUMBER    */
// 05-18-99 late hack///                    
// 05-18-99 late hack///                    g_lpTempPtr--;
// 05-18-99 late hack///
// 05-18-99 late hack///                    memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
// 05-18-99 late hack///                    g_CurrentEquate.Name = malloc(strlen(g_CurrentIdentifier)+1);
// 05-18-99 late hack///                    strcpy(g_CurrentEquate.Name, g_CurrentIdentifier);
// 05-18-99 late hack///                    g_CurrentEquate.Value = g_SegmentOrg[D_SEG];
// 05-18-99 late hack///                    InsertEquate();
// 05-18-99 late hack///
// 05-18-99 late hack///                    g_NumberToResolve = ResolveNumber(g_lpTempPtr);
// 05-18-99 late hack///
// 05-18-99 late hack///                    switch(toupper(yytext[1]))
// 05-18-99 late hack///                    {
// 05-18-99 late hack///                      case 'S':
// 05-18-99 late hack///                        for(g_Index = 0; g_Index < g_NumberToResolve; g_Index++)
// 05-18-99 late hack///                        {
// 05-18-99 late hack///                          if(g_ReservedData[g_SegmentOrg[D_SEG]])
// 05-18-99 late hack///                            yyerror("Overlap detected in Direct Segment");
// 05-18-99 late hack///
// 05-18-99 late hack///                          g_ReservedData[g_SegmentOrg[D_SEG]++] = 1;
// 05-18-99 late hack///                        }
// 05-18-99 late hack///                      break;
// 05-18-99 late hack///
// 05-18-99 late hack///                      default:
// 05-18-99 late hack///                      break;
// 05-18-99 late hack///                    }
// 05-18-99 late hack
                    if(!g_InIncludeFile)
                      //fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);
                      fprintf(g_lpOutFile, "%s", yytext);

                  }

{BitStorage}.*    { /* BitStorage */
                    if(g_DebugOutput)
                      printf("UnitStorage[%s] ",yytext);

// 05-18-99 late hack                    g_lpTempPtr = yytext;
// 05-18-99 late hack                    
// 05-18-99 late hack                    strcpy(g_TempString, yytext);                    
// 05-18-99 late hack                    while(!isspace(*g_lpTempPtr++)); /* move past Identifier */
// 05-18-99 late hack                    g_TempString[g_lpTempPtr - yytext - 1] = 0;
// 05-18-99 late hack
// 05-18-99 late hack                    while(isspace(*g_lpTempPtr++));  /* move to BIT          */
// 05-18-99 late hack                    while(!isspace(*g_lpTempPtr++)); /* move past BIT        */
// 05-18-99 late hack                    while(isspace(*g_lpTempPtr++));  /* move to Idendifier   */
// 05-18-99 late hack                    g_lpTempPtr--;
// 05-18-99 late hack
// 05-18-99 late hack                    strcpy(g_CurrentIdentifier, g_lpTempPtr);
// 05-18-99 late hack
// 05-18-99 late hack                    g_Index = 0;
// 05-18-99 late hack                    while(g_CurrentIdentifier[g_Index] != '.')
// 05-18-99 late hack                      g_Index++;
// 05-18-99 late hack                    
// 05-18-99 late hack                    g_CurrentIdentifier[g_Index] = 0;
// 05-18-99 late hack
// 05-18-99 late hack                    if(((LPEQUATE_LIST)g_lpVoidPtr = IsEquate(g_CurrentIdentifier)))
// 05-18-99 late hack                    { 
// 05-18-99 late hack                      while(*g_lpTempPtr++ != '.');
// 05-18-99 late hack
// 05-18-99 late hack                      switch(((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Type)
// 05-18-99 late hack                      {
// 05-18-99 late hack                        case EQUATE_VALUE:
// 05-18-99 late hack                          g_NumberToResolve = ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Value;
// 05-18-99 late hack                          memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
// 05-18-99 late hack                          g_CurrentEquate.Name = malloc(strlen(g_TempString)+1);
// 05-18-99 late hack
// 05-18-99 late hack                          if(!g_CurrentEquate.Name)
// 05-18-99 late hack                            yyerror("Memory allocation error");
// 05-18-99 late hack
// 05-18-99 late hack                          strcpy(g_CurrentEquate.Name, g_TempString);
// 05-18-99 late hack
// 05-18-99 late hack                          /* now we need to get the bit address from the byte address */
// 05-18-99 late hack                          if(g_NumberToResolve < 0x80)
// 05-18-99 late hack                          { /* this is in scratch pad land */
// 05-18-99 late hack                            g_CurrentEquate.Value = (g_NumberToResolve - 0x20) * 8 + (*g_lpTempPtr - '0');
// 05-18-99 late hack                          }
// 05-18-99 late hack                          else
// 05-18-99 late hack                          { /* this is in SFR land */
// 05-18-99 late hack                            g_CurrentEquate.Value = g_NumberToResolve + (*g_lpTempPtr - '0');
// 05-18-99 late hack                          }
// 05-18-99 late hack                          
// 05-18-99 late hack                          InsertEquate();
// 05-18-99 late hack                        break;
// 05-18-99 late hack
// 05-18-99 late hack                        case EQUATE_REFERENCE: // this may need to go
// 05-18-99 late hack                          if(!g_InIncludeFile)
// 05-18-99 late hack                            fprintf(g_lpOutFile, "%s%s", ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Reference, g_lpTempPtr-1);
// 05-18-99 late hack                      }
// 05-18-99 late hack                    }
// 05-18-99 late hack
                    if(!g_InIncludeFile)
                      //fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);
                      fprintf(g_lpOutFile, "%s", yytext);
                  }

{MultiStorage}.*  { /* MultiStorage */ 
                    if(g_DebugOutput)
                      printf("MultiStorage[%s] ",yytext);
                    
                    if(g_InIncludeFile && !g_ParsingMultiStorage)
                    {
                      g_ParsingMultiStorage = TRUE;
                      
                      if(g_InIncludeFile)
                        fprintf(g_lpOutFile, "\n");
                    }

                    strcpy(g_TempString, yytext);

                    /* remove comments */
                    g_lpTempPtr2 = strchr(g_TempString, ';');

                    if(g_lpTempPtr2)
                      *g_lpTempPtr2 = 0;

                    g_lpTempPtr = strtok(g_TempString, ", \t");

                    fprintf(g_lpOutFile, "%s ", g_lpTempPtr);

                    /* move past db/dw */ 
                    if(g_lpTempPtr)
                    {
                      g_lpTempPtr += strlen(g_lpTempPtr) + 1;

                      while((*g_lpTempPtr == ' ')||(*g_lpTempPtr == '\t'))
                        g_lpTempPtr++;

                      switch(*g_lpTempPtr)
                      {
                        case '\"': 
                        case '\'':
                        break;

                        default:
                          //g_lpTempPtr--;
                          g_lpTempPtr = strtok(NULL, ", \t");
                        break;
                      }
                    }

///                    /* move past db/dw */ 
///                    if(g_lpTempPtr)
///                      //g_lpTempPtr = strtok(NULL, ", \t");
///                      g_lpTempPtr += strlen(g_lpTempPtr) + 1;

                    g_Index = 0;

                    /* break the string into separate units and process any equates */
                    while(g_lpTempPtr != NULL)
                    {
                      if(g_Index++)
                        fprintf(g_lpOutFile, ", ");

if(*g_lpTempPtr == '\"')
{
  g_lpTempPtr = strtok(NULL, "\"");
  fprintf(g_lpOutFile, "\"%s\"", g_lpTempPtr);
  // what about unterminated strings??????
}
else if(*g_lpTempPtr == '\'')
{
  g_lpTempPtr = strtok(NULL, "\'");
  fprintf(g_lpOutFile, "\"%s\"", g_lpTempPtr);
  // what about unterminated strings??????
}
else
{
                      if(((LPEQUATE_LIST)g_lpVoidPtr = IsEquate(g_lpTempPtr)))
                      {
                        if(((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Type == EQUATE_VALUE)
                          fprintf(g_lpOutFile, "0%0Xh", ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Value);
                        else
                          fprintf(g_lpOutFile, "%s", g_lpTempPtr);
                      }
                      else
                        fprintf(g_lpOutFile, "%s", g_lpTempPtr);

}

                      g_lpTempPtr = strtok(NULL, ", \t");
                    }

                    /* now print the comment */
                    if(g_lpTempPtr2)
                      fprintf(g_lpOutFile, ";%s", (char *)g_lpTempPtr2+1);

//                    fprintf(g_lpOutFile, "%s", yytext);
                  }

{SegSelect}{Separator}*{NewLine} { /* SegSelect */
                    yyless(yyleng-1);
                     
                    if(g_DebugOutput)
                      printf("Seg[%s] ", yytext);
                    
                    switch(toupper(*yytext))
                    {
                      case 'C':
                        g_CurrSegType = C_SEG;
                      break;

                      case 'D':
                        g_CurrSegType = D_SEG;
                      break;

                      case 'X':
                        g_CurrSegType = X_SEG;
                      break;

                      case 'I':
                        g_CurrSegType = I_SEG;
                      break;

                      case 'B':
                        g_CurrSegType = B_SEG;
                      break;
                    }

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);  
                  }

{SegSelectAt}     { /* SegSelectAt */ 
                    if(g_DebugOutput)
                      printf("SegSelectAt[%s] ",yytext);
                    
                    g_lpTempPtr = yytext;
                    
                    while(!isspace(*g_lpTempPtr++)); /* move past segtype */
                    while(isspace(*g_lpTempPtr++));  /* move to AT        */
                    while(!isspace(*g_lpTempPtr++)); /* move past AT      */
                    while(isspace(*g_lpTempPtr++));  /* move to NUMBER    */
                    g_lpTempPtr--;

                    switch(toupper(*yytext))
                    {
                      case 'C':
                        g_CurrSegType = C_SEG;
                      break;

                      case 'D':
                        g_CurrSegType = D_SEG;
                      break;

                      case 'X':
                        g_CurrSegType = X_SEG;
                      break;

                      case 'I':
                        g_CurrSegType = I_SEG;
                      break;

                      case 'B':
                        g_CurrSegType = B_SEG;
                      break;
                    }
                    
                    g_SegmentOrg[g_CurrSegType] = ResolveNumber(g_lpTempPtr);
                    
                    if(g_CurrSegType == C_SEG)
                      fprintf(g_lpOutFile, "ORG %Xh", g_SegmentOrg[g_CurrSegType]);

                    if(g_CurrSegType != D_SEG)
                    {
                      // MAY NEED TO GET RID OF THIS
                      fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);
                    }
                    else
                      fprintf(g_lpOutFile, "%s", yytext);
                  }

{Segment}         { /* Segment */  
                    if(g_DebugOutput)
                      printf("Segment ");

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    return TOKEN_SEGMENT;
                  }

{SegType}         { /* SegType */
                    if(g_DebugOutput)
                      printf("SegmentType[%s] ",yytext);

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    return TOKEN_SEGMENT_TYPE;
                  }

{RSeg}.*          { /* RSeg */
                    if(g_DebugOutput)
                      printf("RSeg ");

                    if(!g_InIncludeFile)
                      //fprintf(g_lpOutFile, "%s", yytext);
                      fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);


                    //return TOKEN_RSEG;
                  }

{Org}             { /* Org */
                    if(g_DebugOutput)
                      printf("Org ");

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    fgetpos(g_lpOutFile, &g_ExpressionPosition);
                    return TOKEN_ORG;
                  }

{Bit}{WhiteSpace}.* { /* Bit */ // -  we are cheating here, this should 
                                //    be followed by a bit address
                    if(g_DebugOutput)
                      printf("Bit ");

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    return TOKEN_BIT;
                  }

{End}             { /* End */ 
                    if(g_DebugOutput)
                      printf("End ");

                    if(!g_InIncludeFile)
                    {
                      fprintf(g_lpOutFile,"\n$END_MODULE\n");
 
                      fprintf(g_lpOutFile, "%s%s\n", g_KillEnd? MACRO_COMMENT_PREFIX: "", yytext);
                    }

                    return TOKEN_END;
                  }

{Name}{WhiteSpace}.* { /* Name */ 

                    if(g_DebugOutput)
                      printf("Name ");

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);
                  }

{And}             { /* And */
                    if(g_DebugOutput)
                      printf("AND ");

                    return TOKEN_AND;
                  }

{Or}              { /* Or */
                    if(g_DebugOutput)
                      printf("OR ");

                    return TOKEN_OR;
                  }

{Shl}             { /* Shl */
                    if(g_DebugOutput)
                      printf("SHL ");

                    return TOKEN_SHL;
                  }

{Shr}             { /* Shr */
                    if(g_DebugOutput)
                      printf("SHR ");

                    return TOKEN_SHR;
                  }

{Not}             { /* Not */
                    if(g_DebugOutput)
                      printf("NOT ");

                    return TOKEN_NOT;
                  }

{High}            { /* High */
                    if(g_DebugOutput)
                      printf("High ");

                    return TOKEN_HIGH;
                  }

{Low}             { /* Low */
                    if(g_DebugOutput)
                      printf("Low ");

                    return TOKEN_LOW;
                  }

"/"{Identifier}   { /* Identifier */
                    if(g_DebugOutput)
                      printf("/ ");

                    yyless(1);

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", "/");

                    return '/';
                  }

{BitDigit}        {
                    if(g_DebugOutput)
                      printf("BitDigit[%s] ",yytext);

                    g_HaveBitDigit  = TRUE;

                    if(g_InEquate)
                    {
                      if(g_NumberToResolve  > 0x7f)
                        g_NumberToResolve += (yytext[1] - '0');
                      else if((g_NumberToResolve > 0x1f) && (g_NumberToResolve < 0x30))
                      {
                        g_NumberToResolve = (g_NumberToResolve - 0x20) * 8 + (yytext[1] - '0');
                      }
                      else
                        yyerror("Not Valid Bit Address");
                    }
                    else if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);
                  }

{Identifier}      { /* Identifier */
                    strcpy(g_CurrentIdentifier, yytext);

                    if(g_DebugOutput)
                      printf("Identifier[%s] ",yytext);

/* BUG BUG BUG BUG BUG BUG BUG BUG BUG BUG BUG */
/*  
    If an equate is already in the 
    list the file pointer gets scrogged. 
    We need to indicate that the equate 
    is already defined. fgetpos and fsetpos
    are the problem. OK I THINK IT'S FIXED
*/
/* BUG BUG BUG BUG BUG BUG BUG BUG BUG BUG BUG */

                    g_LastIdInEquateList  = FALSE;

                    if(((LPEQUATE_LIST)g_lpVoidPtr = IsEquate(yytext)))
                    {
                      g_LastIdInEquateList  = TRUE;

                    if(g_DebugOutput)
                      printf(" IsEquate");

                      switch(((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Type)
                      {
                        case EQUATE_VALUE:
                          if(g_DebugOutput)
                            printf(" Value");

                          g_NumberToResolve = ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Value;
                          fgetpos(g_lpOutFile, &g_ExpressionPosition);
                        break;

                        case EQUATE_REFERENCE:
                          if(g_DebugOutput)
                            printf(" Reference");

                          if(!g_InIncludeFile)
                            fprintf(g_lpOutFile, "%s", ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Reference);
                        return TOKEN_IDENTIFIER;
                      }

                      return TOKEN_NUMBER;
                    }
                    else if((g_CurMacro = IsMacro(yytext)))
                    {
                      if(g_DebugOutput)
                        printf(" Macro");

                      BEGIN S_PARM_EXP; // we will be expanding macros
                    }
                    else
                    {
                      if(g_DebugOutput)
                        printf(" Identifier");

                      /* if we end up in a macro or equate we will need to backup */
                      fgetpos(g_lpOutFile, &g_FilePosition);

                      if(!g_InEquate)
                        if(!g_InIncludeFile)
                          fprintf(g_lpOutFile, "%s", yytext);

                      return TOKEN_IDENTIFIER;
                    }
                  }

{Label}           { /* Label */
                    if(g_DebugOutput)
                      printf("Label [%s] ", yytext);

                    //if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s%s%s",  g_InIncludeFile? "\n": "", yytext, g_InIncludeFile? "\n": "");

                  }

{Number}          { /* Number */
                    if(g_DebugOutput)
                      printf("Number[%s] ",yytext);
                    
                    switch(toupper(yytext[yyleng-1]))
                    {
                      case 'H':
                        g_NumberToResolve = strtol(yytext,&g_lpTempPtr,16);
                      break;

                      case 'B':
                        g_NumberToResolve = strtol(yytext,&g_lpTempPtr,2);
                      break;

                      default:
                        g_NumberToResolve = strtol(yytext,&g_lpTempPtr,10);
                      break;
                    } 

                    fgetpos(g_lpOutFile, &g_ExpressionPosition);
                    return TOKEN_NUMBER;
                  }

{CharString}      { /* CharString */
                    if(g_DebugOutput)
                      printf("CharString[%s] ",yytext);
                    
                    strcpy(g_TempString, yytext);

                    /* extract the value of the character */ 
                    g_NumberToResolve = *strtok(g_TempString,"\'\'");
                    fgetpos(g_lpOutFile, &g_ExpressionPosition);
                    return TOKEN_NUMBER;
                  }

{HashMark}        { /* HashMark */
                    if(g_DebugOutput)
                      printf("HashMark ");

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", "#");
                  }

{Indirect}        { /* Indirect */
                    //////////// TODO: may need to resolve expression
                    if(g_DebugOutput)
                      printf("Indirect[%s] ",yytext);

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    return TOKEN_INDIRECT;
                  }

{Separator}       { /* Separator */
                    if(g_DebugOutput)
                      printf(".");

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);
                  }

{DollarAddr}      { /* Dollar Addr */
                    if(g_DebugOutput)
                      printf("Dollar ");

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    return TOKEN_DOLLAR;
                  }

{Control}         {
                    if(g_DebugOutput)
                      printf("Control[%s] ",yytext);

                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s%s", MACRO_COMMENT_PREFIX, yytext);

                  }

{Equate}          { /* Equate */
                    if(g_DebugOutput)
                      printf("Equate ");

                    g_HaveBitDigit  = FALSE;

                    if(g_LastIdInEquateList)
                    {
                      sprintf(g_TempString,"Error, Attempt to redefine equate \"%s\"",g_CurrentIdentifier);
                      yyerror(g_TempString);
                    }

                    /* back up before last identifier */
                    fsetpos(g_lpOutFile, &g_FilePosition);

                    memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
                    g_CurrentEquate.Name = malloc(strlen(g_CurrentIdentifier)+1);
                    
                    if(!g_CurrentEquate.Name)
                      yyerror("Memory allocation error");

                    strcpy(g_CurrentEquate.Name, g_CurrentIdentifier);

                    g_InEquate  =  TRUE;
                    return TOKEN_EQUATE;
                  }

{Comma}           { /* Comma */
                    if(g_DebugOutput)
                      printf("Comma ");
  
                    if(!g_InIncludeFile)
                      fprintf(g_lpOutFile, "%s", yytext);

                    return TOKEN_COMMA;
                  }

{NewLine}         { /* newline */
                    if(g_DebugOutput)
                      printf("Line %d\n",g_LineNumber);

                    if(!g_InIncludeFile || (g_InIncludeFile && g_ParsingMultiStorage))
                    {
                      fprintf(g_lpOutFile, "%s", yytext);
                      g_ParsingMultiStorage = FALSE;
                    }

                    g_LineNumber++;
                    return TOKEN_NEWLINE;
                  }

{Other}           { /* Other  */
                    if(g_DebugOutput)
                      printf("Other[%s] ",yytext);
                    
                    //if(!g_InIncludeFile)
                      //if(!strcmp(yytext,"/"))// this should not be necessary but removing it broke on parens
                        //fprintf(g_lpOutFile, "%s", yytext);

                    return *yytext;
                  }

<S_PARM_EXP>{NewLine} { /* parm newline */
                    if(g_DebugOutput)
                      printf("Line %d\n",g_LineNumber);

                    // we should have a parameter list
                    // that looks like ",param1,param2,..."
                    WriteMacro(g_CurMacro, yytext);
                    // be carefull. we really need strtok for two
                    BEGIN INITIAL;
                    g_LineNumber++;
                  }


<S_PARM_EXP>.* {/* parm .* */
                    if(g_DebugOutput)
                      printf("Line %d\n",g_LineNumber);

                    // we should have a parameter list
                    // that looks like ",param1,param2,..."
                    WriteMacro(g_CurMacro, yytext);
                    // be carefull. we really need strtok for two
                    BEGIN INITIAL;
                  }

<S_MACRO>{EndMacro} { /* EndMacro */
                    if(g_DebugOutput)
                      printf("EndMacro ");
                    
                    sprintf(g_TempString,"; End Macro %s\n",g_CurrentMacro.Name);
                    strcat(g_CurrentMacro.Body, g_TempString);

                    g_InMacro = FALSE;
                    BEGIN S_INITIAL;
                    return TOKEN_ENDMACRO;
                  }

<S_MACRO>{Param}  { /* Param */
                    if(g_DebugOutput)
                      printf("Param ");

                    g_InParamList = TRUE;
                    return TOKEN_PARAM;
                  }

<S_MACRO>{Local}  { /* Local */
                    if(g_DebugOutput)
                      printf("Local ");

                    g_InParamList = FALSE;
                    g_InLocalList = TRUE;
                    return TOKEN_LOCAL;
                  }

<S_MACRO>{Label}  { /* Label */
                    if(g_DebugOutput)
                      printf("Label [%s] ", yytext);

                    strcpy(g_CurrentIdentifier, yytext);
                    
                    if(!(g_lpTempPtr  = strstr(g_CurrentIdentifier, ":")))
                    {
                      sprintf(g_CurrentIdentifier,"Error, invalid local label %s",yytext);
                      yyerror(g_CurrentIdentifier);
                    }
                    
                    *g_lpTempPtr = 0;

                    if((g_lpTempPtr  = strstr(g_CurrentMacro.Locals, g_CurrentIdentifier)))
                    {
                      if(strlen(g_CurrentMacro.Body))
                      {
                        g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                    
                        if(!g_lpTempReference)
                          yyerror("Memory allocation error");

                        g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                        g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                      
                        if(!g_lpTempReference->Reference.Pointer)
                          yyerror("Memory allocation error");

                        strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                        InsertReference(g_lpTempReference);

                        g_CurrentMacro.Body[0] = 0;
                      }

                      g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                  
                      if(!g_lpTempReference)
                        yyerror("Memory allocation error");

                      g_lpTempReference->Reference.Type     = REFERENCE_LOCAL;
                      g_lpTempReference->Reference.Index    = 0;
                      g_lpTempReference->Reference.Pointer  = g_lpTempPtr;
                      InsertReference(g_lpTempReference);
                      strcpy(g_CurrentMacro.Body,":");
                    }
                    else
                    {
                      sprintf(g_CurrentIdentifier,"Error, invalid local label %s",yytext);
                      yyerror(g_CurrentIdentifier);
                    }
                  }

<S_MACRO>{BitDigit} {
                    sprintf(g_TempString, "%s", yytext); 
                    strcat(g_CurrentMacro.Body, g_TempString);
                  }

<S_MACRO>{Identifier} { /* Identifier */
                    strcpy(g_CurrentIdentifier, yytext);

                    if(g_InParamList)
                    {
                      strcat(g_CurrentMacro.Params, yytext);
                      strcat(g_CurrentMacro.Params, ",");
                      g_CurrentMacro.ParamCount++;
                    }
                    else if(g_InLocalList)
                    {
                      strcat(g_CurrentMacro.Locals, yytext);
                      strcat(g_CurrentMacro.Locals, ",");
                      g_CurrentMacro.LocalCount++;
                    }
                    else
                    {
                      strncpy(g_TempString2, g_CurrentMacro.Params, sizeof(g_TempString2)-1);
                      g_TempString2[sizeof(g_TempString2)] = 0;

                      if(
                         (g_lpTempPtr = strstr(g_TempString2,  yytext)) &&
                         (!strcmp(yytext, strtok(g_lpTempPtr, ","))) 
                         //(!strncmp(yytext, g_lpTempPtr, strlen(yytext)))

                        )
                      {
                        if(strlen(g_CurrentMacro.Body))
                        {
                          g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                        
                          if(!g_lpTempReference)
                            yyerror("Memory allocation error");

                          g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                          g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                          
                          if(!g_lpTempReference->Reference.Pointer)
                            yyerror("Memory allocation error");

                          strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                          InsertReference(g_lpTempReference);

                          g_CurrentMacro.Body[0] = 0;
                        }

                        g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                      
                        if(!g_lpTempReference)
                          yyerror("Memory allocation error");

                        g_lpTempReference->Reference.Type     = REFERENCE_PARAM;
                        g_lpTempReference->Reference.Index    = 0;
                        g_lpTempReference->Reference.Pointer  = g_lpTempPtr;
                        InsertReference(g_lpTempReference);
                      }
                      else if((g_lpTempPtr = IsLocal(yytext)))
                      {
                        if(strlen(g_CurrentMacro.Body))
                        {
                          g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                        
                          if(!g_lpTempReference)
                            yyerror("Memory allocation error");

                          g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                          g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                          
                          if(!g_lpTempReference->Reference.Pointer)
                            yyerror("Memory allocation error");

                          strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                          InsertReference(g_lpTempReference);

                          g_CurrentMacro.Body[0] = 0;
                        }

                        g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                      
                        if(!g_lpTempReference)
                          yyerror("Memory allocation error");

                        g_lpTempReference->Reference.Type     = REFERENCE_LOCAL;
                        g_lpTempReference->Reference.Index    = 0;
                        g_lpTempReference->Reference.Pointer  = g_lpTempPtr;
                        InsertReference(g_lpTempReference);
                      }
                      else if(((LPEQUATE_LIST)g_lpVoidPtr = IsEquate(yytext)))
                      {
                        switch(((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Type)
                        {
                          case EQUATE_VALUE:
                            g_NumberToResolve = ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Value;

                            // terminate the current text
                            if(strlen(g_CurrentMacro.Body))
                            {
                              g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                      
                              if(!g_lpTempReference)
                                yyerror("Memory allocation error");

                              g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                              g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                        
                              if(!g_lpTempReference->Reference.Pointer)
                                yyerror("Memory allocation error");

                              strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                              InsertReference(g_lpTempReference);

                              g_CurrentMacro.Body[0] = 0;
                            }

                            // now set up a place holder for the expression
                            g_lpHoldReference = (g_lpTempReference = malloc(sizeof(REFERENCE_LIST)));
                    
                            if(!g_lpHoldReference)
                              yyerror("Memory allocation error");

                            g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                            g_lpTempReference->Reference.Pointer  = NULL;
                      
                            /* Error check here!!!!!!!! */

                            InsertReference(g_lpTempReference);

                            g_CurrentMacro.Body[0] = 0;

//                            sprintf(g_TempString, "%04X", g_NumberToResolve); 
//                            strcat(g_CurrentMacro.Body, g_TempString);
                          break;

                          case EQUATE_REFERENCE:
                            sprintf(g_TempString, "%s", ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Reference); 
                            strcat(g_CurrentMacro.Body, g_TempString);
                          break;
                        }

                        return TOKEN_NUMBER;
                      }
                      else if(((LPMACRO_LIST)g_lpVoidPtr = IsMacro(yytext)))
                      {
/////////////////////////////
                        if(strlen(g_CurrentMacro.Body))
                        {
                          g_lpTempReference = malloc(sizeof(REFERENCE_LIST));

                          if(!g_lpTempReference)
                            yyerror("Memory allocation error");

                          g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                          g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                          
                          if(!g_lpTempReference->Reference.Pointer)
                            yyerror("Memory allocation error");

                          strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                          InsertReference(g_lpTempReference);

                          g_CurrentMacro.Body[0] = 0;
                        }

                        g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                      
                        if(!g_lpTempReference)
                          yyerror("Memory allocation error");

                        g_lpTempReference->Reference.Type     = REFERENCE_MACRO;
                        g_lpTempReference->Reference.Index    = 0;
                        g_lpTempReference->Reference.Pointer  = g_lpVoidPtr;
                        InsertReference(g_lpTempReference);
                        g_ProcessingNestedMacro = TRUE;
/*
                        strcat(g_CurrentMacro.Body,"\n; Begin Macro - ");
                        strcat(g_CurrentMacro.Body, yytext);
                        strcat(g_CurrentMacro.Body, ((LPMACRO_LIST)g_lpVoidPtr)->Macro.Body);
                        strcat(g_CurrentMacro.Body,"\n; End Macro");
*/
/////////////////////////////
                      }
                      else
                      {
                        strcat(g_CurrentMacro.Body, yytext);
                      }
                    }

                    if(g_DebugOutput)
                      printf("Identifier[%s] ",yytext);

                    return TOKEN_IDENTIFIER;
                  }

<S_MACRO>{Comma}  { /* Comma */
                    if(!g_InLocalList)
                      strcat(g_CurrentMacro.Body, yytext);

                    if(g_DebugOutput)
                      printf("Comma ");
  
                    return TOKEN_COMMA;
                  }

<S_MACRO>{Comment} {/* Comment */
                    strcat(g_CurrentMacro.Body, yytext);

                    if(g_DebugOutput)
                      printf("Comment[%s] ",yytext);
                  }

<S_MACRO>{Number} { /* Number */
                    if(g_DebugOutput)
                      printf("Number[%s] ",yytext);
                    
                    switch(toupper(yytext[yyleng-1]))
                    {
                      case 'H':
                        g_NumberToResolve = strtol(yytext,&g_lpTempPtr,16);
                      break;

                      case 'B':
                        g_NumberToResolve = strtol(yytext,&g_lpTempPtr,2);
                      break;

                      default:
                        g_NumberToResolve = strtol(yytext,&g_lpTempPtr,10);
                      break;
                    } 

                   // terminate the current text
                    if(strlen(g_CurrentMacro.Body))
                    {
                      g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
              
                      if(!g_lpTempReference)
                        yyerror("Memory allocation error");

                      g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                      g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                
                      if(!g_lpTempReference->Reference.Pointer)
                        yyerror("Memory allocation error");

                      strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                      InsertReference(g_lpTempReference);

                      g_CurrentMacro.Body[0] = 0;
                    }

                    // now set up a place holder for the expression
                    g_lpHoldReference = (g_lpTempReference = malloc(sizeof(REFERENCE_LIST)));
            
                    if(!g_lpHoldReference)
                      yyerror("Memory allocation error");

                    g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                    g_lpTempReference->Reference.Pointer  = NULL;
              
                    /* Error check here!!!!!!!! */

                    InsertReference(g_lpTempReference);

                    g_CurrentMacro.Body[0] = 0;
                    return TOKEN_NUMBER;
                  }

<S_MACRO>{CharString} { /* CharString */

                    if(g_DebugOutput)
                      printf("CharString[%s] ",yytext);
                    
                    strcpy(g_TempString, yytext);

                    /* extract the value of the character */ 
                    // terminate the current text
                    if(strlen(g_CurrentMacro.Body))
                    {
                      g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
              
                      if(!g_lpTempReference)
                        yyerror("Memory allocation error");

                      g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                      g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                
                      if(!g_lpTempReference->Reference.Pointer)
                        yyerror("Memory allocation error");

                      strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                      InsertReference(g_lpTempReference);

                      g_CurrentMacro.Body[0] = 0;
                    }

                    // now set up a place holder for the expression
                    g_lpHoldReference = (g_lpTempReference = malloc(sizeof(REFERENCE_LIST)));
            
                    if(!g_lpHoldReference)
                      yyerror("Memory allocation error");

                    g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                    g_lpTempReference->Reference.Pointer  = NULL;
              
                    /* Error check here!!!!!!!! */

                    InsertReference(g_lpTempReference);

                    g_CurrentMacro.Body[0] = 0;
                    g_NumberToResolve = *strtok(g_TempString,"\'\'");
                    return TOKEN_NUMBER;
                  }

<S_MACRO>{HashMark} {

                    if(g_DebugOutput)
                      printf("HashMark ");

                    strcat(g_CurrentMacro.Body, "#");
                  }

<S_MACRO>{Indirect} { /* Indirect */
                    //////////// TODO: may need to resolve expression
                    strcat(g_CurrentMacro.Body, yytext);

                    if(g_DebugOutput)
                      printf("Indirect[%s] ",yytext);

                    return TOKEN_INDIRECT;
                  }

<S_MACRO>{Separator} {	/* Separator */
                    strcat(g_CurrentMacro.Body, yytext);

                    if(g_DebugOutput)
                      printf(".");
                  }

<S_MACRO>{DollarAddr} {
                    strcat(g_CurrentMacro.Body, yytext);

                    if(g_DebugOutput)
                      printf("Dollar ");

                    return TOKEN_DOLLAR;
                  }

<S_MACRO>{NewLine}  {	/* NewLine */
                    if(!(g_InParamList || g_InLocalList || g_InMacroHeader))
                      strcat(g_CurrentMacro.Body, yytext);

                    g_InMacroHeader = g_InParamList = g_InLocalList = FALSE;

                    if(g_ProcessingNestedMacro)
                    {
                      g_ProcessingNestedMacro = FALSE;

                      if(strlen(g_CurrentMacro.Body))
                      {
                        g_lpTempReference = malloc(sizeof(REFERENCE_LIST));
                      
                        if(!g_lpTempReference)
                          yyerror("Memory allocation error");

                        g_lpTempReference->Reference.Type     = REFERENCE_TEXT;
                        g_lpTempReference->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
                        
                        if(!g_lpTempReference->Reference.Pointer)
                          yyerror("Memory allocation error");

                        strcpy(g_lpTempReference->Reference.Pointer, g_CurrentMacro.Body);
                        InsertReference(g_lpTempReference);

                        g_CurrentMacro.Body[0] = 0;
                      }
                    }

                    if(g_DebugOutput)
                      printf("Line %d\n",g_LineNumber);

                    g_LineNumber++;
                    return TOKEN_NEWLINE;
                  }

<S_MACRO>{Other}  { /* Other  */
                    strcat(g_CurrentMacro.Body, yytext);

                    if(g_DebugOutput)
                      printf("Other[%s] ",yytext);
                    
                    return *yytext;
                  }

%%
/*  Program Section  */ 
/***************************************************************************/
/***************************************************************************/
void  InitIncludeStack(void)
{
  memset(&g_FileStack,0,sizeof(g_FileStack));
}
/***************************************************************************/
BOOL  PushIncludeFile(char* p_lpFileName)
{
  FILE  *l_TempFile;
  char  l_TempBuff[80];
  int   filelength(int);

  if(g_FileStack.StackPointer == (MAX_INCLUDE - 1))
    return  FALSE;

  if(!(l_TempFile = fopen(p_lpFileName, "r")))
  {
    sprintf(l_TempBuff, "Cannot find file \"%s\".",p_lpFileName);
    
    if(!g_FileStack.StackPointer)
      printf("%s\n", l_TempBuff);
    else
      yyerror(l_TempBuff);

    return FALSE;
  }

  if(g_FileStack.StackPointer)
  {
    if(!strcmp(p_lpFileName,
               GetCurrentFileName()))
    {
      sprintf(g_CurrentIdentifier,"Include file recurrsion (%s) detected!",p_lpFileName);
      yyerror("Include file recurrsion detected!");
      fclose(l_TempFile);
      return FALSE;
    }
  }
 
  g_FileStack.List[g_FileStack.StackPointer].BufferState  = YY_CURRENT_BUFFER;
  g_FileStack.List[g_FileStack.StackPointer].FilePointer  = l_TempFile;
  g_FileStack.List[g_FileStack.StackPointer].LineNumber   = g_LineNumber;
  g_LineNumber = 1;
  yyin = l_TempFile;

  if(g_FileStack.StackPointer)
  {
#ifdef WIN32
    yy_switch_to_buffer(yy_create_buffer(yyin, filelength(fileno(l_TempFile))));
#else
    yy_switch_to_buffer(yy_create_buffer(yyin, YY_BUF_SIZE));
#endif
  }

  strcpy(g_FileStack.List[g_FileStack.StackPointer++].FileName, p_lpFileName);

  g_InIncludeFile = (g_FileStack.StackPointer > 1)? TRUE: FALSE;

  if(g_DebugOutput)
    printf("\nEntering file \"%s\"\n", p_lpFileName);

  return TRUE;
}
/***************************************************************************/
BOOL  PopIncludeFile()
{
  g_ParsingMultiStorage = FALSE;

  if(g_FileStack.StackPointer == 0)
    return FALSE;
  
  fclose(yyin);

  yyin = g_FileStack.List[--g_FileStack.StackPointer].FilePointer;

  if(g_FileStack.StackPointer)
  {
    yy_switch_to_buffer(g_FileStack.List[g_FileStack.StackPointer].BufferState);
  }
  else
    g_InIncludeFile = FALSE;

  g_LineNumber  = g_FileStack.List[g_FileStack.StackPointer].LineNumber;

  g_InIncludeFile = (g_FileStack.StackPointer > 1)? TRUE: FALSE;

  if(g_DebugOutput)
    printf("\nExiting to file \"%s\"\n", GetCurrentFileName());

  return g_FileStack.StackPointer? TRUE: FALSE;
}
/***************************************************************************/
const char  *GetCurrentFileName(void)
{
  return  g_FileStack.List[g_FileStack.StackPointer-1].FileName;
}
/***************************************************************************/
FILE  *GetRootFile(void)
{
  return g_FileStack.List[0].FilePointer;
}
/***************************************************************************/
char *ExtractIncludeFileName()
{
  char  l_Separators[]  = "()";
  char  *l_Token;
  
  strcpy(g_TempString, yytext);
  l_Token = strtok(g_TempString, l_Separators);

  if(l_Token != NULL)
    l_Token = strtok(NULL, l_Separators);

  return l_Token;
}
/***************************************************************************/
int ResolveNumber(char *p_NumberString)
{
  char  l_TempChar;
  char  *l_lpTempPtr  = p_NumberString;
  int   l_Value       = 0;

  switch(toupper(l_lpTempPtr[strlen(l_lpTempPtr)-1]))
  {
    case 'H':
      while((l_TempChar = toupper(*l_lpTempPtr++)) != 'H')
      {
        l_Value <<= 4;

        if((l_TempChar <= 'F') && (l_TempChar >= 'A'))
          l_Value += (l_TempChar - 'A' + 0x0A);
        else
          l_Value += (l_TempChar - '0');
      }
    break;

    case 'B':
      while(*l_lpTempPtr && (l_TempChar = toupper(*l_lpTempPtr++)) != 'D')
      {
        l_Value <<= 1;
        l_Value += (l_TempChar - '0');
      }
    break;

    case 'D':
    default:
      while(*l_lpTempPtr && (l_TempChar = toupper(*l_lpTempPtr++)) != 'D')
      {
        l_Value *= 10;
        l_Value += (l_TempChar - '0');
      }
    break;
  }

  return l_Value;
}
/***************************************************************************/
BOOL  IsParam(char *p_lpString)
{
  char  l_Separators[]  = ",";
  char  *l_Token;

  strcpy(g_TempString, g_CurrentMacro.Params);
  l_Token = strtok(g_TempString, l_Separators);

  while(l_Token != NULL)
  {
    if(!StrCmp(l_Token, p_lpString))
    {
      strcpy(g_TempString, l_Token);
      return TRUE;
    }

    l_Token = strtok(NULL, l_Separators);
  }

  return FALSE;
}
/***************************************************************************/
char *IsLocal(char *p_lpString)
{
  char  *l_lpToken,
        *l_lpLocal,
        *l_lpBuff;

  if(!(l_lpLocal = strstr(g_CurrentMacro.Locals, p_lpString)))
    return NULL;

  if(!(l_lpBuff = malloc(strlen(l_lpLocal)+1)))
  {
    yyerror("Memory Allocation Error");
    return NULL;
  }

  strcpy(l_lpBuff, l_lpLocal);

  l_lpToken = strtok(l_lpBuff," ,\t:");

  while(l_lpToken)
  {
    if(!StrCmp(l_lpToken, p_lpString))
      break;

    l_lpToken = strtok(NULL," ,\t:");
  }

  if(!l_lpToken)
    l_lpLocal = NULL;

  free(l_lpBuff);
  return  l_lpLocal;
}
/***************************************************************************/
int  StrCmp(char *p_S1, char *p_S2)
{
  int l_i,
      l_Length = strlen(p_S1) > strlen(p_S2)? strlen(p_S1): strlen(p_S2),
      l_Difference;

  
  for(l_i = 0;  l_i < l_Length; l_i++)  
  {
    if((l_Difference = toupper(p_S1[l_i]) - toupper(p_S2[l_i])))
      break;
  }

  return l_Difference;
}
/***************************************************************************/
#define EQUATE_HASH_SIZE 1000
#define MACRO_HASH_SIZE 500

LPEQUATE_LIST g_EquateListTable[EQUATE_HASH_SIZE];
LPMACRO_LIST  g_MacroListTable[MACRO_HASH_SIZE];
/***************************************************************************/
char *StrToUpper(unsigned char *p_lpString)
{
  unsigned char *l_lpString = p_lpString;

  if(p_lpString)
    while(*l_lpString)
      *l_lpString++ = toupper(*l_lpString);

  return p_lpString;
}
/***************************************************************************/
#define HASH(KEY, SIZE) Hash((KEY)) % (SIZE) 
/***************************************************************************/
unsigned int Hash(unsigned char *p_lpKey)// ElfHash
{
  unsigned int h = 0;
  unsigned int g;

  //p_lpKey = StrToUpper(p_lpKey);

  while(*p_lpKey)
  {
    h = (h << 4) + toupper(*p_lpKey++);

    if((g = h & 0xf0000000))
      h ^= g >> 24;

    h &= ~g;
  }

  return h;
}
/***************************************************************************/
LPMACRO_LIST IsMacro(char *p_lpIdentifier)
{
  int l_Index = HASH(p_lpIdentifier, MACRO_HASH_SIZE);
  LPMACRO_LIST l_Temp = g_MacroListTable[l_Index];

  while(l_Temp)
  {
    if(!StrCmp(p_lpIdentifier,  l_Temp->Macro.Name))
      break;

    l_Temp = l_Temp->Next;
  }

  return l_Temp;
}
/***************************************************************************/
BOOL  InsertReference(LPREFERENCE_LIST p_lpReference)
{
  int               l_i     = 0;
  LPREFERENCE_LIST  l_lpCur = g_CurrentMacro.References;
  char              *l_lpTemp, 
                    *l_lpTempNext;

  if(!l_lpCur)
  {
    g_CurrentMacro.References       = p_lpReference;
    g_CurrentMacro.References->Next = NULL;
    l_lpCur                         = g_CurrentMacro.References;
  }
  else
  {
    while(l_lpCur->Next)
      l_lpCur = l_lpCur->Next;

    l_lpCur->Next = p_lpReference;
    l_lpCur       = l_lpCur->Next;
    l_lpCur->Next = NULL;
  }

  switch(l_lpCur->Reference.Type)  
  {
    case REFERENCE_PARAM:
//      if(l_lpCur->Reference.Pointer != g_CurrentMacro.Params)
if(strncmp(l_lpCur->Reference.Pointer, g_CurrentMacro.Params, strlen(l_lpCur->Reference.Pointer)))
      {
        l_lpTemp  = g_CurrentMacro.Params;

        while(l_i++ < g_CurrentMacro.ParamCount)
        {
          l_lpTempNext  = strstr(l_lpTemp,  ",");
          l_lpTemp      = l_lpTempNext+1;

//          if(l_lpCur->Reference.Pointer == l_lpTemp)
//            break;
if(!strncmp(l_lpCur->Reference.Pointer, l_lpTemp, strlen(l_lpCur->Reference.Pointer)))
  break;
        } 
      }

      l_lpCur->Reference.Index    = l_i;
      l_lpCur->Reference.Pointer  = NULL;
    break;

    case REFERENCE_LOCAL:
      if(l_lpCur->Reference.Pointer != g_CurrentMacro.Locals)
      {
        l_lpTemp  = g_CurrentMacro.Locals;

        while(l_i++ < g_CurrentMacro.LocalCount)
        {
          l_lpTempNext  = strstr(l_lpTemp,  ",");
          l_lpTemp      = l_lpTempNext+1;

          if(l_lpCur->Reference.Pointer == l_lpTemp)
            break;
        } 
      }

      l_lpCur->Reference.Index    = l_i;
      l_lpCur->Reference.Pointer  = NULL;
    break;

    case REFERENCE_TEXT:
      l_lpCur->Reference.Index  = 0;
    break;
  }

  return TRUE;    
}
/***************************************************************************/
/* this routine will generate a table of pointers for the macro body that
 * looks like this:
 *
 [*]-->body start
 [*]-->param or local instance
 [*]-->body next
 [*]-->param or local instance
  ...
 [*]-->param or local instance
 [*]-->body last
 *
 * TODO: embed pointers to embedded macros instead of text
 *       currently embedded macros cannot have params or locals
*/
BOOL  InsertMacro()
{
  char          *l_lpNewBody;
  int l_Index = HASH(g_CurrentMacro.Name, MACRO_HASH_SIZE);
  LPMACRO_LIST  l_Temp;

  /* insert macro into a table - HERE */
  if(!(l_Temp = malloc(sizeof(MACRO_LIST))))
  {
    yyerror("Memory allocation error");
    return  FALSE;
  }

  /* get that last chunk of text */ 
  if(strlen(g_CurrentMacro.Body))
  {
    l_lpNewBody = malloc(sizeof(REFERENCE_LIST));
  
    if(!l_lpNewBody)
      yyerror("Memory allocation error");

    ((LPREFERENCE_LIST)l_lpNewBody)->Reference.Type     = REFERENCE_TEXT;
    ((LPREFERENCE_LIST)l_lpNewBody)->Reference.Pointer  = malloc(strlen(g_CurrentMacro.Body)+1);
    
    if(!((LPREFERENCE_LIST)l_lpNewBody)->Reference.Pointer)
      yyerror("Memory allocation error");

    strcpy(((LPREFERENCE_LIST)l_lpNewBody)->Reference.Pointer, g_CurrentMacro.Body);
    InsertReference((LPREFERENCE_LIST)l_lpNewBody);
  }

  memcpy(&l_Temp->Macro, &g_CurrentMacro, sizeof(g_CurrentMacro));

  free(g_CurrentMacro.Body);
  l_Temp->Macro.Body = NULL;

  l_Temp->Macro.PointerCount  = l_Temp->Macro.ParamInstances  + 
                                l_Temp->Macro.LocalInstances  +
                                l_Temp->Macro.MacroInstances  +
                                l_Temp->Macro.ParamCount  +
                                l_Temp->Macro.LocalCount  + 
                                l_Temp->Macro.MacroCount  + 1;

  l_Temp->Next  = g_MacroListTable[l_Index];
  g_MacroListTable[l_Index]   = l_Temp;
  
  /* now split the params and locals */
  l_lpNewBody = strtok(l_Temp->Macro.Params,", \t");

  while(l_lpNewBody)
    l_lpNewBody = strtok(NULL,", \t");

  l_lpNewBody = strtok(l_Temp->Macro.Locals,", \t");

  while(l_lpNewBody)
    l_lpNewBody = strtok(NULL,", \t");

  g_CurrentMacro.Name   = NULL;
  g_CurrentMacro.Body   = NULL;
  g_CurrentMacro.Params = NULL;
  g_CurrentMacro.Locals = NULL;
  return TRUE;
}
/***************************************************************************/
void  InitMacros()
{
  memset(&g_MacroListTable, 0, sizeof(g_MacroListTable));
//  g_MacroList           = NULL;
  g_CurrentMacro.Name   = NULL;
  g_CurrentMacro.Body   = NULL;
  g_CurrentMacro.Params = NULL;
  g_CurrentMacro.Locals = NULL;
  
  /* this doesn't really belong here */
  memset(g_ReservedData, 0, sizeof(g_ReservedData));
}
/***************************************************************************/
void  FreeMacros()
{
  int i;
  LPMACRO_LIST      l_Temp;
  LPREFERENCE_LIST  l_lpCurReference;
//return;
/* CLEAN UP REFERENCES !!!!!!!! I THINK I GOT 'EM */

  if(g_CurrentMacro.Name)
  {
    free(g_CurrentMacro.Name);
    free(g_CurrentMacro.Body);   
    free(g_CurrentMacro.Params); 
    free(g_CurrentMacro.Locals);
     
    g_CurrentMacro.Name   = NULL;
    g_CurrentMacro.Body   = NULL;
    g_CurrentMacro.Params = NULL;
    g_CurrentMacro.Locals = NULL;
  }
  
//  if(!g_MacroList)
//    return;

  if(g_DebugOutput)
    printf("\nMacros parsed:\n");

  for(i = 0; i < MACRO_HASH_SIZE; i++)
  {
    while(g_MacroListTable[i])
    {
      l_Temp = g_MacroListTable[i];

      if(g_DebugOutput)
      {
        printf("%40s\n", l_Temp->Macro.Name);      
      }

      l_Temp            = g_MacroListTable[i];
      l_lpCurReference  = l_Temp->Macro.References;

      while(l_lpCurReference)
      {
        if(l_lpCurReference->Reference.Type == REFERENCE_TEXT)
        {
          free(l_lpCurReference->Reference.Pointer);
          l_lpCurReference->Reference.Pointer = NULL; /* may want to remove for efficiency */
        }

        l_lpCurReference = l_lpCurReference->Next;
        free(l_Temp->Macro.References);
        l_Temp->Macro.References  = l_lpCurReference;
      }

      g_MacroListTable[i] = g_MacroListTable[i]->Next;
      free(l_Temp->Macro.Name);
      free(l_Temp->Macro.Body);   
      free(l_Temp->Macro.Params);
free(l_Temp->Macro.Locals); 
      free(l_Temp);
    }
  }
}
/***************************************************************************/
void  WriteMacro(LPMACRO_LIST p_lpMacroEntry, char *p_lpParams)
{
  int               l_i;
  char              *l_lpTemp,
                    *l_lpTemp2;
  LPREFERENCE_LIST  l_lpCurReference = p_lpMacroEntry->Macro.References;
  char              l_lpParams[256],
                    l_lpMacroPrefix[256];
 
  if(strlen(p_lpMacroEntry->Macro.Locals))
  {
    if(!(l_lpTemp = strrchr(GetCurrentFileName(),*DIRECTORY_SEPARATOR)))
      l_lpTemp = (char *)GetCurrentFileName();

    sprintf(l_lpMacroPrefix, 
            "%s_%04x_", 
            l_lpTemp,
            g_LabelCount++);

    if((l_lpTemp = strrchr(l_lpMacroPrefix,'.')))
      *l_lpTemp = '_';
  }

  fprintf(g_lpOutFile,"\n");
    
  while(l_lpCurReference)
  {
    switch(l_lpCurReference->Reference.Type)
    {
      case REFERENCE_TEXT:
        fprintf(g_lpOutFile, "%s", l_lpCurReference->Reference.Pointer);
      break;

      case REFERENCE_PARAM:        
        strcpy(l_lpParams, p_lpParams);
        l_lpTemp = strtok(l_lpParams,", \t;\r");

if(!l_lpTemp)
{
sprintf(l_lpParams, "Macro %s, Parameter missing",((*p_lpMacroEntry).Macro).Name);
yyerror(l_lpParams);
}

        if(*l_lpTemp == ';') /* this is a comment */
          break;

        for(l_i = 0; l_i < l_lpCurReference->Reference.Index; l_i++)
        {
          l_lpTemp = strtok(NULL, ", \t;\r");

          if(*l_lpTemp == ';') /* this is a comment */
          {
            l_i = -1;
            break;
          }
        }

        if(l_i == -1)
          break;


/////
        if(((LPEQUATE_LIST)g_lpVoidPtr = IsEquate(l_lpTemp)))
        {
          if(((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Type == EQUATE_VALUE)
            fprintf(g_lpOutFile, "0%0Xh", ((LPEQUATE_LIST)g_lpVoidPtr)->Equate.Value);
        }
        else
/////
        fprintf(g_lpOutFile, "%s", l_lpTemp);
      break;

      case REFERENCE_LOCAL:// ???
        l_lpTemp = p_lpMacroEntry->Macro.Locals;

        for(l_i = 0; l_i < l_lpCurReference->Reference.Index; l_i++)
          l_lpTemp += strlen(l_lpTemp) + 1;

        fprintf(g_lpOutFile, "%s%s", l_lpMacroPrefix,l_lpTemp);
      break;

      case REFERENCE_MACRO:
        /* if the macro has parameters, they will be in the next text reference */

        if(((LPMACRO_LIST)l_lpCurReference->Reference.Pointer)->Macro.ParamCount)
        {
// YY_HACK 
yyerror("Error, Nested macros with parameters are not supported at this time\n");
return;

          if(l_lpCurReference->Next->Reference.Type != REFERENCE_TEXT)
          {
            yyerror("Error, invalid macro parameter list");
            return;
          }

          l_lpTemp  = l_lpCurReference->Next->Reference.Pointer;
          strcpy(l_lpParams, l_lpTemp);

          /* macro parameters are delimited by commas */
          l_lpTemp2 = strchr(l_lpParams, '\n'); 
          l_lpTemp  = strchr(l_lpParams, ',');

          if((!l_lpTemp) ||(l_lpTemp2 < l_lpTemp))
          {  
            yyerror("Error, invalid macro parameter list");
            return;
          }

          for(l_i = 0; l_i < ((LPMACRO_LIST)l_lpCurReference->Reference.Pointer)->Macro.ParamCount; l_i++)
            l_lpTemp = strchr(l_lpTemp, ',');

          *l_lpTemp = 0;
//*l_lpTemp2 = 0;
        }
/*
        if(((LPMACRO_LIST)l_lpCurReference->Reference.Pointer)->Macro.ParamCount)
        {
          l_lpTemp  = malloc(strlen(l_lpCurReference->Next->Reference.Pointer) - strlen(l_lpParams) + 1);

          if(!l_lpTemp)
            yyerror("Memory allocation error");

          strcpy(l_lpTemp, (char *)l_lpCurReference->Next->Reference.Pointer + strlen(l_lpParams));
          free(l_lpCurReference->Next->Reference.Pointer);
          l_lpCurReference->Next->Reference.Pointer = l_lpTemp;
        }
*/
        WriteMacro(l_lpCurReference->Reference.Pointer, l_lpCurReference->Next->Reference.Pointer);/*l_lpParams);*/

        /* if there are parameters they follow the macro and should not be printed */
        if(((LPMACRO_LIST)l_lpCurReference->Reference.Pointer)->Macro.ParamCount)
          l_lpCurReference = l_lpCurReference->Next;
      break;
    }

    l_lpCurReference = l_lpCurReference->Next;
  }
}
/***************************************************************************/
LPEQUATE_LIST IsEquate(char *p_lpIdentifier)
{
  int l_Index = HASH(p_lpIdentifier, EQUATE_HASH_SIZE);
  LPEQUATE_LIST l_Temp = g_EquateListTable[l_Index];

  while(l_Temp)
  {
    if(!StrCmp(p_lpIdentifier,  l_Temp->Equate.Name))
      break;

    l_Temp = l_Temp->Next;
  }

  return l_Temp;
}
/***************************************************************************/
BOOL  InsertEquate()
{
  int l_Index = HASH(g_CurrentEquate.Name, EQUATE_HASH_SIZE);
  LPEQUATE_LIST l_Temp;

  /* insert macro into a table - HERE */
  if(!(l_Temp = malloc(sizeof(EQUATE_LIST))))
  {
    if(!l_Temp)
      yyerror("Memory allocation error");

    return  FALSE;
  }

  memcpy(&l_Temp->Equate, &g_CurrentEquate, sizeof(g_CurrentEquate));
  l_Temp->Next  = g_EquateListTable[l_Index];
  g_EquateListTable[l_Index]  = l_Temp;

  g_CurrentEquate.Name = NULL;
  return TRUE;
}
/***************************************************************************/
void  InitEquates()
{
  memset(&g_EquateListTable, 0, sizeof(g_EquateListTable));
//  g_EquateList  = NULL;
  g_CurrentEquate.Name  = NULL;
  g_LastIdInEquateList  = FALSE;

  /*  pre-init known equates */
  /* P0 */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("P0")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "P0");
  g_CurrentEquate.Value = 0x80;
  InsertEquate();

  /* PCON */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("PCON")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "PCON");
  g_CurrentEquate.Value = 0x87;
  InsertEquate();

  /* TCON */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("TCON")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "TCON");
  g_CurrentEquate.Value = 0x88;
  InsertEquate();

  /* P1 */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("P1")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "P1");
  g_CurrentEquate.Value = 0x90;
  InsertEquate();

  /* SCON0 */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("SCON0")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "SCON0");
  g_CurrentEquate.Value = 0x98;
  InsertEquate();

  /* P2 */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("P2")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "P2");
  g_CurrentEquate.Value = 0xA0;
  InsertEquate();

  /* IE */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("IE")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "IE");
  g_CurrentEquate.Value = 0xA8;
  InsertEquate();

  /* P3 */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("P3")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "P3");
  g_CurrentEquate.Value = 0xB0;
  InsertEquate();

  /* IP */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("IP")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "IP");
  g_CurrentEquate.Value = 0xB8;
  InsertEquate();

  /* SCON1 */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("SCON1")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "SCON1");
  g_CurrentEquate.Value = 0xC0;
  InsertEquate();

  /* T2CON */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("T2CON")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "T2CON");
  g_CurrentEquate.Value = 0xC8;
  InsertEquate();

  /* PSW */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("PSW")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "PSW");
  g_CurrentEquate.Value = 0xD0;
  InsertEquate();

  /* WDCON */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("WDCON")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "WDCON");
  g_CurrentEquate.Value = 0xD8;
  InsertEquate();

  /* ACC */
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("ACC")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "ACC");
  g_CurrentEquate.Value = 0xE0;
  InsertEquate();

  /* EIE */
  
  memset(&g_CurrentEquate, 0, sizeof(g_CurrentEquate));
  g_CurrentEquate.Name  = malloc(strlen("EIE")+1);

  if(!g_CurrentEquate.Name)
    yyerror("Memory allocation error");

  strcpy(g_CurrentEquate.Name, "EIE");
  g_CurrentEquate.Value = 0xE8;
  InsertEquate();
}
/***************************************************************************/
/* TODO: We have major memory leaks here. Link lists and all associated 
 *       malloced fields need to be fixed. I think they are fixed!
 *
 */
void  FreeEquates()
{
  int i;
  LPEQUATE_LIST l_Temp;

  if(g_CurrentEquate.Name)
  {
    free(g_CurrentEquate.Name);
    g_CurrentEquate.Name = NULL;
  }
  
//  if(!g_EquateList)
//    return;

  if(g_DebugOutput)
    printf("\nEquates parsed:\n");

  for(i = 0; i < EQUATE_HASH_SIZE; i++)
  {
    while(g_EquateListTable[i])
    {
      l_Temp = g_EquateListTable[i];

      if(g_DebugOutput)
        printf("%40s\tEQU\t%07Xh\n", l_Temp->Equate.Name, l_Temp->Equate.Value);

      g_EquateListTable[i] = g_EquateListTable[i]->Next;
    
      if(l_Temp->Equate.Reference)
        free(l_Temp->Equate.Reference);

      free(l_Temp->Equate.Name);
      free(l_Temp);
    }
  }
}
/***************************************************************************/
/****************************** end my lex source **************************/
/***************************************************************************/
