/* ----------------------------------------------------------------------
 * FILE: main.c
 * PACKAGE: as31 - 8031/8051 Assembler.
 *
 * DESCRIPTION:
 *  The file contains main(). It handles the arguments and makes
 *  sure that pass 1 is done before pass 2 etc...
 *
 * REVISION HISTORY:
 *  Jan. 19, 1990 - Created. (Ken Stauffer)
 *    August 1999 - Converted to 24 bit addressing. (Tom Chenot, Dallas Semiconductor) 
 *                - Fixed 1-off bug in certain opcodes.
 *                - Added Intel Hex386 output generation.
 *                - Replaced lexical analyzer.
 *                - Added TBin output generation.
 *                - Added TLib output generation.
 *
 *
 * AUTHOR:
 *  Original code in this file written by Ken Stauffer (University of Calgary).
 *  January, 1990. 
 *  24 bit addressing extensions written by Tom Chenot (Dallas Semiconductor).
 *  August 1999.
 *
 */

/*
 $Workfile: main.c $
 $Revision: 12 $
 $Date: 11/16/00 10:01a $
 $Author: Chenot $
 $Modtime: 11/16/00 10:01a $
 $Log: /TINI/390 Assembler/source/a390/main.c $
 * 
 * 12    11/16/00 10:01a Chenot
 * 
 * 11    11/16/00 8:05a Chenot
 * 
 * 10    9/09/00 6:28p Chenot
 * 
 * 9     9/08/00 12:52p Chenot
 * 
 * 8     9/08/00 10:04a Chenot
 * 
 * 6     5/20/00 6:16p Chenot
 * 
 * 5     5/20/00 3:20p Chenot
 * 
 * 4     5/18/00 9:25p Chenot
 * 
 * 3     5/16/00 8:46p Chenot
 * 
 * 2     5/15/00 6:10a Chenot
 * 
 * 30    4/07/00 7:23a Chenot
 * 
 * 29    4/07/00 6:56a Chenot
 * 
 * 28    10/01/99 6:54a Tomc
 * 
 * 27    9/23/99 8:02a Tomc
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <setjmp.h>

#ifdef WIN32
#define DIR_SEP '\\'
#else
#define DIR_SEP '/'
#endif

extern int           lineno;
extern int           pass,
                     fatal;
extern unsigned long lc,
                     cur_lc;
extern int           yydebug;
extern FILE          *yyin;
extern int           g_DebugOutput;
extern char          g_StatsOn;

extern int  yyparse(void);
extern void yyrestart(FILE *input_file);
extern void DumpSymbols(void);
extern char WriteStats(void);
extern void Cleanup(void);
extern void DumpFixups(void);
extern void DumpStringTable(void);
extern void emitusage(void);
extern void emitopen(char *file, char *ftype, char *arg);
extern void emitclose(void);
extern void emitdelete(void);
extern void emitaddr(unsigned long int a);
extern void syminit(void);
extern void InitPass(void);
extern void GenerateSymbolEquateFile(char *lpFileName, char *lpBanner);

extern void DumpExtraStatistics(void);

/**/
char    g_Native            = 1;
char    g_ErrorsEncountered = 0;
jmp_buf main_env;
char    *asmfile;
int     dashl               = 0;
char    *dashS              = NULL;
FILE    *listing;
int     g_EndOfFile         = 0;
char    g_BannerTopString[] = 
#ifdef WIN32
          "ͻ\n";
#else
          "+------------------------------------------------------------------+\n";
#endif
char    g_BannerString[] =
#ifdef WIN32
          "";
#else
          "|";
#endif
char    g_BannerBottomString[] = 
#ifdef WIN32
          "ͼ\n";
#else
          "+------------------------------------------------------------------+\n";
#endif

#define DEFAULT_EXTENSION   "tlib"
#define ALTERNATE_EXTENSION "tbin"

#define VERSION_MAJOR 0
#define VERSION_MINOR 110
#define BUILD "$Revision: 12 $"
void PrintBanner()
{
  char  l_lpBuild[]      = BUILD;
  char  *l_lpTemp        = NULL;

  fprintf(stderr,
          "\n%s%s Dallas Semiconductor DS80C390 Assembler, Beta Version %d.%03d.%04u %s\n%s"
          "Last build %s at %s\n\n",
          g_BannerTopString,
          g_BannerString,
          VERSION_MAJOR, 
          VERSION_MINOR, 
          strtol(strtok(l_lpBuild, "evision: $R"), &l_lpTemp, 10),
          g_BannerString, 
          g_BannerBottomString,
          __DATE__, 
          __TIME__);
  if(dashS)
    printf("Generating linkage file tini.inc with banner \"%s\"\n", dashS);
}
void main(int argc, char **argv)
{
//  char  *defaultOutFmt   = DEFAULT_EXTENSION;
  FILE  *fin             = NULL;
  char  *p               = NULL,
        *dashF           = NULL,
        dashFAlternate[] = ALTERNATE_EXTENSION,
        *dashA           = NULL;
  char  objfile[260]     = "";
  char  lstfile[260]     = "";
  int   i                = 0;
  int   l_Count          = 0;
  char  l_DisplayLineNo  = 1;
  char  l_VerboseMode    = 0;
  char  l_QuietMode      = 0;

  if(argc < 2)
  {
    PrintBanner();
    fprintf(stderr,
            "Usage: %s [-l] [-s] [-Ffmt] <input file>\n\t-l Generate list file \t\t(default: no list file)\n\t-s Print statistical information(default: off)\n\t-d Allow direct declarations\n\t-F Output format \t\t(default: tlib)",
            strrchr(argv[0], DIR_SEP)? strrchr(argv[0], DIR_SEP)+1: argv[0]);
    emitusage();
    fprintf(stderr, "\n");
    exit(1);
  }
  
  InitPass();

  for(i = 1; i < argc; i++ ) 
  {
    if(argv[i][0] != '-') 
      break;

    if(toupper(argv[i][1]) == 'V')
      l_VerboseMode = 1;
    else if(argv[i][1] == 'l') 
      dashl = 1;
    else if(argv[i][1] == 's') 
      g_StatsOn = 1;
    else if(argv[i][1] == 'q')
      l_QuietMode = 1;
    else if(argv[i][1] == 'd')
      g_Native = 0;
    else if(argv[i][1] == 'S')
    {
      dashS = &argv[i][2];
    }
    else if(dashF == NULL && argv[i][1] == 'F')
      dashF = argv[i]+2;
    else if(dashA == NULL && argv[i][1] == 'A')
      dashA = argv[i]+2;
    else 
    {
      PrintBanner();
      fprintf(stderr,"*** Duplicate or unknown flag.\n");
      fprintf(stderr, "\n");
      exit(1);
    }
  }

  if(i == argc) 
  {
    PrintBanner();
    fprintf(stderr, "*** Missing input file.\n");
    fprintf(stderr, "\n");
    exit(1);
  }

  if(!(p = strchr(argv[i], '.')))
  {
    PrintBanner();
    fprintf(stderr, "*** Error parsing command line\n");
    fprintf(stderr, "\n");
    exit(1);
  }

  if(!l_QuietMode)
    PrintBanner();

  asmfile = argv[i];

  if((yyin = (fin = fopen(argv[i], "r"))) == NULL) 
  {
    fprintf(stderr, "*** Cannot open input file: %s\n", argv[i]);
    fprintf(stderr, "\n");
    exit(1);
  }

  if(dashl) 
  {
    strcpy(lstfile, argv[i]);
    strcpy(lstfile + (p - argv[i]),".lst");
    listing = fopen(lstfile, "w");
  
    if(listing == NULL) 
    {
      fprintf(stderr, "Cannot open file: %s for writting.\n", lstfile);
      exit(1);
    }
  }

  strcpy(objfile, argv[i]);
  strcpy(objfile + (p - argv[i]),".");
  dashF? strcat(objfile, dashF): g_Native? strcat(objfile, DEFAULT_EXTENSION): 
                                           (strcat(objfile, ALTERNATE_EXTENSION),
                                             dashF = dashFAlternate);

  emitopen(objfile, dashF, dashA);

  syminit();
  fatal   = 0;
  lineno  = 1;
  pass    = 0;
  cur_lc  = lc = 0x0000;
   
  if(setjmp(main_env)) 
  {
    emitclose();
    fclose(fin);
    unlink(objfile);
    exit(1);
  }

  /*
   * P A S S    1
   */
  g_EndOfFile = 0;
  if(!l_QuietMode)
    fprintf(stderr,"Starting pass 1...\n");

  do
  { /* none of that recursive stuff, do parse iteratively */
    /* our source blows the top off of the stack */
    if(yyparse())    /* parse a statement   */
    {
      break;
    }
    
    if(l_VerboseMode) /* this slows execution */
    {
      if(l_DisplayLineNo)
        fprintf(stderr,"Line: %d\r", lineno);
      else
      {
        switch((l_Count++)%4)
        {
          case 0:
            fprintf(stderr,"|\r");
          break;

          case 1:
            fprintf(stderr,"/\r");
          break;

          case 2:
            fprintf(stderr,"-\r");
          break;

          case 3:
            fprintf(stderr,"\\\r");
          break;
        }

        l_Count &= 3;
      }
    }
  } while (!g_EndOfFile);

  if(fatal)
    longjmp(main_env,1);

  rewind(fin);
  yyrestart(yyin);
  lineno  = 1;
  pass++;
  cur_lc  = lc  = 0x0000;
  emitaddr(lc);

  /*
   * P A S S    2
   */
  g_EndOfFile = 0;

  if(!l_QuietMode)
    fprintf(stderr,"Starting pass 2...\n");
//yydebug=1;

  do
  { /* none of that recursive stuff, do parse iteratively */
    /* our source blows the top off of the stack */
    if(yyparse())    /* parse a statement   */
    {
      break;
    }

    if(l_VerboseMode) /* this slows execution */
    {
      if(l_DisplayLineNo)
        fprintf(stderr,"Line: %d\r", lineno);
      else
      {
        switch((l_Count++)%4)
        {
          case 0:
            fprintf(stderr,"|\r");
          break;

          case 1:
            fprintf(stderr,"/\r");
          break;

          case 2:
            fprintf(stderr,"-\r");
          break;

          case 3:
            fprintf(stderr,"\\\r");
          break;
        }

        l_Count &= 3;
      }
    }
  } while (!g_EndOfFile);

  if(l_DisplayLineNo)
    if(!l_QuietMode)
      fprintf(stderr,"Lines parsed: %d\n", lineno);
  
  if(dashS)
    GenerateSymbolEquateFile("tini.inc", dashS);
  
  emitclose();
  fclose(fin);

  if(dashl)
  {
    DumpSymbols();
    fclose(listing);
  }

  if(g_ErrorsEncountered)
  {
    fprintf(stderr,"*** Errors detected ***\n");
    emitdelete();
  }
  else
  {
    WriteStats();
  }

  exit(0);
}
/***************************************************************
 * yywrap
 */
int   yywrap  (void)
{
  g_EndOfFile = 1;
  return 1;
}
